# -*- coding: utf-8 -*-
# Author: TDC Team
# License: MIT
from packaging import version
import pkg_resources
"""This file contains all metadata of datasets in TDC.

Attributes:
    adme_dataset_names (list): all adme dataset names
    admet_benchmark (dict): a dictionary with key the TDC task and value a list of dataset names
    admet_metrics (dict): a dictionary with key the dataset name and value the recommended metric
    admet_splits (dict): a dictionary with key the dataset name and value the recommended split
    antibodyaff_dataset_names (list): all antibody_aff dataset names
    benchmark2id (dict): benchmark names to dataverse download ID
    benchmark2type (dict): benchmark names to file type in download format
    benchmark_names (dict): a dictionary mapping benchmark group name to each benchmark group dataset names
    bm_metric_names (dict): a dictionary mapping benchmark group name to each benchmark group metric names
    bm_split_names (dict): a dictionary mapping benchmark group name to each benchmark group split names
    catalyst_dataset_names (list): all catalyst dataset names
    category_names (dict): mapping from ML problem (1st tier) to all tasks
    crisproutcome_dataset_names (list): all crispr outcome dataset names
    dataset_list (list): total list of dataset names in TDC
    dataset_names (dict): mapping from task name to list of dataset names
    ddi_dataset_names (list): all ddi dataset names
    develop_dataset_names (list): all develop dataset names
    distribution_oracles (list): all distribution learning oracles, i.e. molecule evaluators
	docking_oracles (list): all docking oracles, i.e. RMSD
    docking_benchmark (dict): docking benchmark target names
    docking_target_info (dict): docking benchmark target pockets info
    download_oracle_names (list): oracle names that require downloading predictors
    drugres_dataset_names (list): all drugres dataset names
    drugsyn_benchmark (dict): drugcombo benchmark group targets
    drugsyn_dataset_names (list): all drugsyn dataset names
    drugsyn_metrics (dict): a dictionary with key the dataset name and value the recommended metric
    drugsyn_splits (dict):  a dictionary with key the dataset name and value the recommended split
    dti_dataset_names (list): all dti dataset names
    dti_dg_benchmark (dict): dti_dg benchmark group dataset names
    dti_dg_metrics (dict): a dictionary with key the dataset name and value the recommended metric
    dti_dg_splits (dict):  a dictionary with key the dataset name and value the recommended split
    epitope_dataset_names (list): all epitope dataset names
    evaluator_name (list): list of evaluator names
    forwardsyn_dataset_names (list): all reaction dataset names
    gda_dataset_names (list): all gda dataset names
    generation_datasets (list): all generation dataset names
    guacamol_oracle (list): list of oracles from guacamol
    hts_dataset_names (list): all hts dataset names
    meta_oracle_name (list): list of all meta oracle names
    molgenpaired_dataset_names (list): all molgenpaired dataset names
    mti_dataset_names (list): all mti dataset names
    name2id (dict): mapping from dataset names to dataverse id
    name2stats (dict): mapping from dataset names to statistics
    name2type (dict): mapping from dataset names to downloaded file format
    oracle2id (dict): mapping from oracle names to dataverse id
    oracle2type (dict): mapping from oracle names to downloaded file format
    receptor2id (dict): mapping from receptor id to dataverse id
    oracle_names (list): list of all oracle names
    paired_dataset_names (list): all paired dataset names
    paratope_dataset_names (list): all paratope dataset names
    peptidemhc_dataset_names (list): all peptidemhc dataset names
    ppi_dataset_names (list): all ppi dataset names
    property_names (list): a list of oracles that correspond to some molecular properties
    qm_dataset_names (list): all qm dataset names
    retrosyn_dataset_names (list): all retrosyn dataset names
    sdf_file_names (list): list of sdf file names
    single_molecule_dataset_names (list): all molgen dataset names
	multiple_molecule_dataset_names (list): all ligandmolgen dataset names
    synthetic_oracle_name (list): all oracle names for synthesis
    test_multi_pred_dataset_names (list): test multi pred task name
    test_single_pred_dataset_names (list): test single pred task name
    toxicity_dataset_names (list): all toxicity dataset names
    trivial_oracle_names (list): a list of oracle names for trivial oracles
    yield_dataset_names (list): all yield dataset names
"""
####################################
# test cases
test_single_pred_dataset_names = ["test_single_pred"]
test_multi_pred_dataset_names = ["test_multi_pred"]

# single_pred prediction

toxicity_dataset_names = [
    "tox21",
    "toxcast",
    "clintox",
    "herg_karim",
    "herg",
    "herg_central",
    "dili",
    "skin_reaction",
    "ames",
    "carcinogens_lagunin",
    "ld50_zhu",
]

adme_dataset_names = [
    "lipophilicity_astrazeneca", "solubility_aqsoldb",
    "hydrationfreeenergy_freesolv", "caco2_wang", "pampa_ncats",
    "approved_pampa_ncats", "hia_hou", "pgp_broccatelli", "bioavailability_ma",
    "vdss_lombardo", "cyp2c19_veith", "cyp2d6_veith", "cyp3a4_veith",
    "cyp1a2_veith", "cyp2c9_veith", "cyp2c9_substrate_carbonmangels",
    "cyp2d6_substrate_carbonmangels", "cyp3a4_substrate_carbonmangels",
    "bbb_martins", "b3db_classification", "b3db_regression", "ppbr_az",
    "half_life_obach", "clearance_hepatocyte_az", "clearance_microsome_az",
    "hlm", "rlm"
]

hts_dataset_names = [
    "hiv", "sarscov2_3clpro_diamond", "sarscov2_vitro_touret",
    "orexin1_receptor_butkiewicz", "m1_muscarinic_receptor_agonists_butkiewicz",
    "m1_muscarinic_receptor_antagonists_butkiewicz",
    "potassium_ion_channel_kir2.1_butkiewicz",
    "kcnq2_potassium_channel_butkiewicz",
    "cav3_t-type_calcium_channels_butkiewicz", "choline_transporter_butkiewicz",
    "serine_threonine_kinase_33_butkiewicz",
    "tyrosyl-dna_phosphodiesterase_butkiewicz"
]

qm_dataset_names = ["qm7", "qm7b", "qm8", "qm9"]

epitope_dataset_names = ["iedb_jespersen", "pdb_jespersen"]

paratope_dataset_names = ["sabdab_liberis"]

develop_dataset_names = ["tap", "sabdab_chen"]

# multi_pred prediction

dti_dataset_names = [
    "davis",
    "kiba",
    "bindingdb_kd",
    "bindingdb_ic50",
    "bindingdb_ki",
    "bindingdb_patent",
]

ppi_dataset_names = ["huri"]

peptidemhc_dataset_names = ["mhc2_iedb_jensen", "mhc1_iedb-imgt_nielsen"]

ddi_dataset_names = ["drugbank", "twosides"]

mti_dataset_names = ["mirtarbase"]

gda_dataset_names = ["disgenet"]

crisproutcome_dataset_names = ["leenay"]

drugres_dataset_names = [
    "gdsc1",
    "gdsc2",
]

drugsyn_dataset_names = ["oncopolypharmacology", "drugcomb"]

antibodyaff_dataset_names = ["protein_sabdab"]

yield_dataset_names = ["uspto_yields", "buchwald-hartwig"]

catalyst_dataset_names = ["uspto_catalyst"]

tcr_epi_dataset_names = ["weber", "panpep"]

trial_outcome_dataset_names = ['phase1', 'phase2', 'phase3']

proteinpeptide_dataset_names = ['brown_mdm2_ace2_12ca5']

cellxgene_dataset_names = [
    "scperturb_drug_AissaBenevolenskaya2021",
    "scperturb_drug_SrivatsanTrapnell2020_sciplex2",
    "scperturb_drug_SrivatsanTrapnell2020_sciplex3",
    "scperturb_drug_SrivatsanTrapnell2020_sciplex4",
    "scperturb_drug_ZhaoSims2021",
    "scperturb_gene_NormanWeissman2019",
    "scperturb_gene_ReplogleWeissman2022_rpe1",
    "scperturb_gene_ReplogleWeissman2022_k562_essential",
]

resource_dataset_names = [
    "opentargets_ra_data_splits",
    "opentargets_ibd_data_splits",
    "opentargets_ra_drug_evidence",
    "opentargets_ibd_drug_evidence",
    "opentargets_ra_data_splits_idx",
    "opentargets_ibd_data_splits_idx",
    "tchard_full",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-0",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-1",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-2",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-3",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-4",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-0",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-1",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-2",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-3",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-4",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-0",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-1",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-2",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-3",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-4",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-0",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-1",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-2",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-3",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-4",
    "tchard_pep_cdr3b_only_neg_assays_test-0",
    "tchard_pep_cdr3b_only_neg_assays_test-1",
    "tchard_pep_cdr3b_only_neg_assays_test-2",
    "tchard_pep_cdr3b_only_neg_assays_test-3",
    "tchard_pep_cdr3b_only_neg_assays_test-4",
    "tchard_pep_cdr3b_only_neg_assays_train-0",
    "tchard_pep_cdr3b_only_neg_assays_train-1",
    "tchard_pep_cdr3b_only_neg_assays_train-2",
    "tchard_pep_cdr3b_only_neg_assays_train-3",
    "tchard_pep_cdr3b_only_neg_assays_train-4",
    "tchard_pep_cdr3b_only_sampled_negs_test-0",
    "tchard_pep_cdr3b_only_sampled_negs_test-1",
    "tchard_pep_cdr3b_only_sampled_negs_test-2",
    "tchard_pep_cdr3b_only_sampled_negs_test-3",
    "tchard_pep_cdr3b_only_sampled_negs_test-4",
    "tchard_pep_cdr3b_only_sampled_negs_train-0",
    "tchard_pep_cdr3b_only_sampled_negs_train-1",
    "tchard_pep_cdr3b_only_sampled_negs_train-2",
    "tchard_pep_cdr3b_only_sampled_negs_train-3",
    "tchard_pep_cdr3b_only_sampled_negs_train-4",
]

resources = {
    "opentargets_dti": {
        "splits": [
            "opentargets_ra_data_splits",
            "opentargets_ibd_data_splits",
        ],
        "datasets": [
            "opentargets_ra_drug_evidence",
            "opentargets_ibd_drug_evidence",
        ],
        "all": [
            "opentargets_ra_data_splits",
            "opentargets_ibd_data_splits",
            "opentargets_ra_drug_evidence",
            "opentargets_ibd_drug_evidence",
        ],
    },
    "tchard": {
        "splits_raw": {
            "train": {
                "tchard_pep_cdr3b_only_neg_assays": {
                    0: "tchard_pep_cdr3b_only_neg_assays_train-0",
                    1: "tchard_pep_cdr3b_only_neg_assays_train-1",
                    2: "tchard_pep_cdr3b_only_neg_assays_train-2",
                    3: "tchard_pep_cdr3b_only_neg_assays_train-3",
                    4: "tchard_pep_cdr3b_only_neg_assays_train-4",
                },
                "tchard_pep_cdr3b_only_sampled_negs_train": {
                    0: "tchard_pep_cdr3b_only_sampled_negs_train-0",
                    1: "tchard_pep_cdr3b_only_sampled_negs_train-1",
                    2: "tchard_pep_cdr3b_only_sampled_negs_train-2",
                    3: "tchard_pep_cdr3b_only_sampled_negs_train-3",
                    4: "tchard_pep_cdr3b_only_sampled_negs_train-4",
                },
                "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train": {
                    0: "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-0",
                    1: "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-1",
                    2: "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-2",
                    3: "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-3",
                    4: "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-4",
                },
                "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train": {
                    0: "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-0",
                    1: "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-1",
                    2: "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-2",
                    3: "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-3",
                    4: "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-4",
                }
            },
            "test": {
                "tchard_pep_cdr3b_only_neg_assays": {
                    0: "tchard_pep_cdr3b_only_neg_assays_test-0",
                    1: "tchard_pep_cdr3b_only_neg_assays_test-1",
                    2: "tchard_pep_cdr3b_only_neg_assays_test-2",
                    3: "tchard_pep_cdr3b_only_neg_assays_test-3",
                    4: "tchard_pep_cdr3b_only_neg_assays_test-4",
                },
                "tchard_pep_cdr3b_only_sampled_negs_train": {
                    0: "tchard_pep_cdr3b_only_sampled_negs_test-0",
                    1: "tchard_pep_cdr3b_only_sampled_negs_test-1",
                    2: "tchard_pep_cdr3b_only_sampled_negs_test-2",
                    3: "tchard_pep_cdr3b_only_sampled_negs_test-3",
                    4: "tchard_pep_cdr3b_only_sampled_negs_test-4",
                },
                "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train": {
                    0: "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-0",
                    1: "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-1",
                    2: "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-2",
                    3: "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-3",
                    4: "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-4",
                },
                "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train": {
                    0: "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-0",
                    1: "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-1",
                    2: "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-2",
                    3: "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-3",
                    4: "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-4",
                }
            },
            "dev": {}  # no dev set on tchard
        },
        "all": ["tchard_full",],
        "config": {
            "Y": "label",
        }
    },
}

####################################
# generation

retrosyn_dataset_names = ["uspto50k", "uspto"]

forwardsyn_dataset_names = ["uspto"]

single_molecule_dataset_names = ["zinc", "moses", "chembl", "chembl_v29"]

multiple_molecule_dataset_names = ["dude", "pdbbind", "scpdb"]  #'crossdock',

paired_dataset_names = ["uspto50k", "uspto"]

####################################
# resource

compound_library_names = [
    "drugbank_drugs",
    "chembl_drugs",
    "broad_repurposing_hub",
    "antivirals",
]
biokg_library_names = ["hetionet"]

####################################
# oracles

#### evaluator for distribution learning, the input of __call__ is list of smiles
distribution_oracles = [
    "novelty",
    "diversity",
    "uniqueness",
    "validity",
    "fcd_distance",
    "kl_divergence",
]

docking_oracles = ["rmsd", "kabsch_rmsd", "smina"]

property_names = [
    "drd2",
    "qed",
    "logp",
    "sa",
    "gsk3b",
    "jnk3",
]

evaluator_name = [
    "roc-auc",
    "f1",
    "pr-auc",
    "precision",
    "recall",
    "accuracy",
    "mse",
    "rmse",
    "mae",
    "r2",
    "micro-f1",
    "macro-f1",
    "kappa",
    "avg-roc-auc",
    "rp@k",
    "pr@k",
    "pcc",
    "spearman",
    "range_logAUC",
]

evaluator_name.extend(distribution_oracles)
evaluator_name.extend(docking_oracles)

guacamol_oracle = [
    "rediscovery",
    "similarity",
    "median",
    "isomers",
    "mpo",
    "hop",
    "celecoxib_rediscovery",
    "troglitazone_rediscovery",
    "thiothixene_rediscovery",
    "aripiprazole_similarity",
    "albuterol_similarity",
    "mestranol_similarity",
    "isomers_c7h8n2o2",
    "isomers_c9h10n2o2pf2cl",
    "isomers_c11h24",
    "osimertinib_mpo",
    "fexofenadine_mpo",
    "ranolazine_mpo",
    "perindopril_mpo",
    "amlodipine_mpo",
    "sitagliptin_mpo",
    "zaleplon_mpo",
    "sitagliptin_mpo_prev",
    "zaleplon_mpo_prev",
    "median1",
    "median2",
    "valsartan_smarts",
    "deco_hop",
    "scaffold_hop",
]

####################################
# Benchmark Datasets

admet_benchmark = {
    "ADME": [
        "caco2_wang",
        "hia_hou",
        "pgp_broccatelli",
        "bioavailability_ma",
        "lipophilicity_astrazeneca",
        "solubility_aqsoldb",
        "bbb_martins",
        "ppbr_az",
        "vdss_lombardo",
        "cyp2d6_veith",
        "cyp3a4_veith",
        "cyp2c9_veith",
        "cyp2d6_substrate_carbonmangels",
        "cyp3a4_substrate_carbonmangels",
        "cyp2c9_substrate_carbonmangels",
        "half_life_obach",
        "clearance_microsome_az",
        "clearance_hepatocyte_az",
    ],
    "Tox": ["herg", "ames", "dili", "ld50_zhu"],
}

drugsyn_benchmark = {
    "Synergy": [
        "drugcomb_css",
        "drugcomb_hsa",
        "drugcomb_loewe",
        "drugcomb_bliss",
        "drugcomb_zip",
    ]
}

dti_dg_benchmark = {"DTI": ["bindingdb_patent"]}

docking_benchmark = {
    "Targets": [
        "1iep",
        "2rgp",
        "3eml",
        "3ny8",
        "4rlu",
        "4unn",
        "5mo4",
        "7l11",
        "3pbl",
    ]
}

docking_target_info = {
    "3pbl": {
        "center": (9, 22.5, 26),
        "size": (15, 15, 15)
    },
    "1iep": {
        "center": (15.61389189189189, 53.38013513513513, 15.454837837837842),
        "size": (15, 15, 15),
    },
    "2rgp": {
        "center": (16.292121212121213, 34.87081818181819, 92.0353030303030),
        "size": (15, 15, 15),
    },
    "3eml": {
        "center": (-9.063639999999998, -7.1446, 55.86259999999999),
        "size": (15, 15, 15),
    },
    "3ny8": {
        "center": (2.2488, 4.68495, 51.39820000000001),
        "size": (15, 15, 15)
    },
    "4rlu": {
        "center": (-0.7359999999999999, 22.75547368421052, -31.2368947368421),
        "size": (15, 15, 15),
    },
    "4unn": {
        "center": (5.684346153846153, 18.191769230769232, -7.37157692307692),
        "size": (15, 15, 15),
    },
    "5mo4": {
        "center": (-44.901709677419355, 20.490354838709674, 8.483354838709678),
        "size": (15, 15, 15),
    },
    "7l11": {
        "center": (-21.814812500000006, -4.216062499999999, -27.983781250000),
        "size": (15, 15, 15),
    },
}

####################################

#### Benchmark Metrics
admet_metrics = {
    "caco2_wang": "mae",
    "hia_hou": "roc-auc",
    "pgp_broccatelli": "roc-auc",
    "bioavailability_ma": "roc-auc",
    "lipophilicity_astrazeneca": "mae",
    "solubility_aqsoldb": "mae",
    "bbb_martins": "roc-auc",
    "ppbr_az": "mae",
    "vdss_lombardo": "spearman",
    "cyp2c9_veith": "pr-auc",
    "cyp2d6_veith": "pr-auc",
    "cyp3a4_veith": "pr-auc",
    "cyp2c9_substrate_carbonmangels": "pr-auc",
    "cyp3a4_substrate_carbonmangels": "roc-auc",
    "cyp2d6_substrate_carbonmangels": "pr-auc",
    "half_life_obach": "spearman",
    "clearance_hepatocyte_az": "spearman",
    "clearance_microsome_az": "spearman",
    "ld50_zhu": "mae",
    "herg": "roc-auc",
    "ames": "roc-auc",
    "dili": "roc-auc",
}

drugsyn_metrics = {
    "drugcomb_css": "mae",
    "drugcomb_hsa": "mae",
    "drugcomb_loewe": "mae",
    "drugcomb_bliss": "mae",
    "drugcomb_zip": "mae",
    "drugcomb_css_brain": "mae",
    "drugcomb_css_ovary": "mae",
    "drugcomb_css_lung": "mae",
    "drugcomb_css_skin": "mae",
    "drugcomb_css_hematopoietic_lymphoid": "mae",
    "drugcomb_css_breast": "mae",
    "drugcomb_css_prostate": "mae",
    "drugcomb_css_kidney": "mae",
    "drugcomb_css_colon": "mae",
}

dti_dg_metrics = {"bindingdb_patent": "pcc"}

#### Benchmark Splits
admet_splits = {
    "caco2_wang": "scaffold",
    "hia_hou": "scaffold",
    "pgp_broccatelli": "scaffold",
    "bioavailability_ma": "scaffold",
    "lipophilicity_astrazeneca": "scaffold",
    "solubility_aqsoldb": "scaffold",
    "bbb_martins": "scaffold",
    "ppbr_az": "scaffold",
    "vdss_lombardo": "scaffold",
    "cyp2c9_veith": "scaffold",
    "cyp2d6_veith": "scaffold",
    "cyp3a4_veith": "scaffold",
    "cyp2c9_substrate_carbonmangels": "scaffold",
    "cyp3a4_substrate_carbonmangels": "scaffold",
    "cyp2d6_substrate_carbonmangels": "scaffold",
    "half_life_obach": "scaffold",
    "clearance_hepatocyte_az": "scaffold",
    "clearance_microsome_az": "scaffold",
    "ld50_zhu": "scaffold",
    "herg": "scaffold",
    "ames": "scaffold",
    "dili": "scaffold",
}

drugsyn_splits = {
    "drugcomb_css": "combination",
    "drugcomb_hsa": "combination",
    "drugcomb_loewe": "combination",
    "drugcomb_bliss": "combination",
    "drugcomb_zip": "combination",
}

dti_dg_splits = {"bindingdb_patent": "group"}

####################################

# evaluator for single molecule, the input of __call__ is a single smiles OR list of smiles
download_oracle_names = [
    "drd2", "gsk3b", "jnk3", "fpscores", "cyp3a4_veith", "smina"
]
# download_oracle_names = ['drd2', 'gsk3b', 'jnk3', 'fpscores', 'cyp3a4_veith']
download_oracle_names = ["drd2", "gsk3b", "jnk3", "fpscores", "cyp3a4_veith"
                        ] + [
                            "drd2_current",
                            "gsk3b_current",
                            "jnk3_current",
                        ]

trivial_oracle_names = ["qed", "logp", "sa"] + guacamol_oracle
synthetic_oracle_name = ["askcos", "ibm_rxn"]
download_receptor_oracle_name = [
    "1iep_docking",
    "2rgp_docking",
    "3eml_docking",
    "3ny8_docking",
    "4rlu_docking",
    "4unn_docking",
    "5mo4_docking",
    "7l11_docking",
    "drd3_docking",
    "3pbl_docking",
    "1iep_docking_normalize",
    "2rgp_docking_normalize",
    "3eml_docking_normalize",
    "3ny8_docking_normalize",
    "4rlu_docking_normalize",
    "4unn_docking_normalize",
    "5mo4_docking_normalize",
    "7l11_docking_normalize",
    "drd3_docking_normalize",
    "3pbl_docking_normalize",
    "1iep_docking_vina",
    "2rgp_docking_vina",
    "3eml_docking_vina",
    "3ny8_docking_vina",
    "4rlu_docking_vina",
    "4unn_docking_vina",
    "5mo4_docking_vina",
    "7l11_docking_vina",
    "drd3_docking_vina",
    "3pbl_docking_vina",
]

meta_oracle_name = [
    "isomer_meta",
    "rediscovery_meta",
    "similarity_meta",
    "median_meta",
    "docking_score",
    "molecule_one_synthesis",
    "pyscreener",
]

oracle_names = (download_oracle_names + trivial_oracle_names +
                distribution_oracles + synthetic_oracle_name +
                meta_oracle_name + docking_oracles +
                download_receptor_oracle_name)

molgenpaired_dataset_names = ["qed", "drd2", "logp"]

generation_datasets = (retrosyn_dataset_names + forwardsyn_dataset_names +
                       molgenpaired_dataset_names +
                       multiple_molecule_dataset_names)
# generation
####################################

category_names = {
    "single_pred": [
        "Tox",
        "ADME",
        "HTS",
        "Epitope",
        "Develop",
        "QM",
        "Paratope",
        "Yields",
        "CRISPROutcome",
    ],
    "multi_pred": [
        "DTI", "PPI", "DDI", "PeptideMHC", "DrugRes", "AntibodyAff", "DrugSyn",
        "MTI", "GDA", "Catalyst", "TCR_Epitope_Binding", "TrialOutcome",
        "CellXGene"
    ],
    "generation": ["RetroSyn", "Reaction", "MolGen"],
}


def get_task2category():
    task2category = {}
    for i, j in category_names.items():
        for x in j:
            task2category[x] = i
    return task2category


dataset_names = {
    "Tox": toxicity_dataset_names,
    "ADME": adme_dataset_names,
    "HTS": hts_dataset_names,
    "DTI": dti_dataset_names,
    "PPI": ppi_dataset_names,
    "DDI": ddi_dataset_names,
    "RetroSyn": retrosyn_dataset_names,
    "Reaction": forwardsyn_dataset_names,
    "MolGen": single_molecule_dataset_names,
    "sbdd": multiple_molecule_dataset_names,
    "PeptideMHC": peptidemhc_dataset_names,
    "Epitope": epitope_dataset_names,
    "Develop": develop_dataset_names,
    "DrugRes": drugres_dataset_names,
    "QM": qm_dataset_names,
    "AntibodyAff": antibodyaff_dataset_names,
    "DrugSyn": drugsyn_dataset_names,
    "MTI": mti_dataset_names,
    "GDA": gda_dataset_names,
    "Paratope": paratope_dataset_names,
    "Yields": yield_dataset_names,
    "Catalyst": catalyst_dataset_names,
    "CRISPROutcome": crisproutcome_dataset_names,
    "test_single_pred": test_single_pred_dataset_names,
    "test_multi_pred": test_multi_pred_dataset_names,
    "TCREpitopeBinding": tcr_epi_dataset_names,
    "TrialOutcome": trial_outcome_dataset_names,
    "ProteinPeptide": proteinpeptide_dataset_names,
    "CellXGene": cellxgene_dataset_names,
    "Resource": resource_dataset_names,
}

benchmark_names = {
    "admet_group": admet_benchmark,
    "drugcombo_group": drugsyn_benchmark,
    "docking_group": docking_benchmark,
    "dti_dg_group": dti_dg_benchmark,
}

bm_metric_names = {
    "admet_group": admet_metrics,
    "drugcombo_group": drugsyn_metrics,
    "dti_dg_group": dti_dg_metrics,
}

bm_split_names = {
    "admet_group": admet_splits,
    "drugcombo_group": drugsyn_splits,
    "dti_dg_group": dti_dg_splits,
}

dataset_list = []
for i in dataset_names.keys():
    dataset_list = dataset_list + [i.lower() for i in dataset_names[i]]

name2type = {
    "toxcast": "tab",
    "tox21": "tab",
    "clintox": "tab",
    "lipophilicity_astrazeneca": "tab",
    "solubility_aqsoldb": "tab",
    "hydrationfreeenergy_freesolv": "tab",
    "caco2_wang": "tab",
    "pampa_ncats": "tab",
    "approved_pampa_ncats": "tab",
    "hia_hou": "tab",
    "pgp_broccatelli": "tab",
    "f20_edrug3d": "tab",
    "f30_edrug3d": "tab",
    "bioavailability_ma": "tab",
    "vd_edrug3d": "tab",
    "cyp2c19_veith": "tab",
    "cyp2d6_veith": "tab",
    "cyp3a4_veith": "tab",
    "cyp1a2_veith": "tab",
    "cyp2c9_veith": "tab",
    "cyp2c9_substrate_carbonmangels": "tab",
    "cyp2d6_substrate_carbonmangels": "tab",
    "cyp3a4_substrate_carbonmangels": "tab",
    "carcinogens_lagunin": "tab",
    "halflife_edrug3d": "tab",
    "clearance_edrug3d": "tab",
    "bbb_adenot": "tab",
    "bbb_martins": "tab",
    "b3db_classification": "tab",
    "b3db_regression": "tab",
    "ppbr_ma": "tab",
    "ppbr_edrug3d": "tab",
    "hiv": "tab",
    "sarscov2_3clpro_diamond": "tab",
    "sarscov2_vitro_touret": "tab",
    "orexin1_receptor_butkiewicz": "tab",
    "m1_muscarinic_receptor_agonists_butkiewicz": "tab",
    "m1_muscarinic_receptor_antagonists_butkiewicz": "tab",
    "potassium_ion_channel_kir2.1_butkiewicz": "tab",
    "kcnq2_potassium_channel_butkiewicz": "tab",
    "cav3_t-type_calcium_channels_butkiewicz": "tab",
    "choline_transporter_butkiewicz": "tab",
    "serine_threonine_kinase_33_butkiewicz": "tab",
    "tyrosyl-dna_phosphodiesterase_butkiewicz": "tab",
    "davis": "tab",
    "kiba": "tab",
    "bindingdb_kd": "tab",
    "bindingdb_ic50": "csv",
    "bindingdb_ki": "csv",
    "bindingdb_patent": "csv",
    "huri": "tab",
    "drugbank": "tab",
    "twosides": "csv",
    "mhc1_iedb-imgt_nielsen": "tab",
    "mhc2_iedb_jensen": "tab",
    "uspto": "csv",
    "uspto50k": "tab",
    "zinc": "tab",
    "moses": "tab",
    "chembl": "tab",
    "chembl_v29": "csv",
    "qed": "tab",
    "drd2": "tab",
    "logp": "tab",
    "drugcomb": "pkl",
    "gdsc1": "pkl",
    "gdsc2": "pkl",
    "iedb_jespersen": "pkl",
    "pdb_jespersen": "pkl",
    "qm7": "pkl",
    "qm7b": "pkl",
    "qm8": "pkl",
    "qm9": "pkl",
    "scpdb": "zip",
    "dude": "zip",
    #  'crossdock': 'zip',
    "tap": "tab",
    "sabdab_chen": "tab",
    "protein_sabdab": "csv",
    "oncopolypharmacology": "pkl",
    "mirtarbase": "csv",
    "disgenet": "csv",
    "sabdab_liberis": "pkl",
    "uspto_yields": "pkl",
    "uspto_catalyst": "csv",
    "buchwald-hartwig": "pkl",
    "hetionet": "tab",
    "herg": "tab",
    "herg_central": "tab",
    "herg_karim": "tab",
    "dili": "tab",
    "ppbr_az": "tab",
    "ames": "tab",
    "skin_reaction": "tab",
    "drugbank_drugs": "csv",
    "clearance_microsome_az": "tab",
    "clearance_hepatocyte_az": "tab",
    "half_life_obach": "tab",
    "ld50_zhu": "tab",
    "vdss_lombardo": "tab",
    "leenay": "tab",
    "test_single_pred": "tab",
    "test_multi_pred": "tab",
    "gdsc_gene_symbols": "tab",
    "weber": "tab",
    "primekg": "tab",
    "primekg_drug_feature": "tab",
    "primekg_disease_feature": "tab",
    "drug_comb_meta_data": "pkl",
    "phase1": "tab",
    "phase2": "tab",
    "phase3": "tab",
    "brown_mdm2_ace2_12ca5": "xlsx",
    "scperturb_drug_AissaBenevolenskaya2021": "h5ad",
    "scperturb_drug_SrivatsanTrapnell2020_sciplex2": "h5ad",
    "scperturb_drug_SrivatsanTrapnell2020_sciplex3": "h5ad",
    "scperturb_drug_SrivatsanTrapnell2020_sciplex4": "h5ad",
    "scperturb_drug_ZhaoSims2021": "h5ad",
    "scperturb_gene_NormanWeissman2019": "h5ad",
    "scperturb_gene_ReplogleWeissman2022_rpe1": "h5ad",
    "scperturb_gene_ReplogleWeissman2022_k562_essential": "h5ad",
    "opentargets_ra_data_splits": "json",
    "opentargets_ra_data_splits_idx": "json",
    "opentargets_ibd_data_splits": "json",
    "opentargets_ibd_data_splits_idx": "json",
    "opentargets_ra_drug_evidence": "tab",
    "opentargets_ibd_drug_evidence": "tab",
    "hlm": "tab",
    "rlm": "tab",
    "tchard_full": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-0": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-1": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-2": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-3": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-4": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-0": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-1": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-2": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-3": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-4": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-0": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-1": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-2": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-3": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-4": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-0": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-1": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-2": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-3": "tab",
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-4": "tab",
    "tchard_pep_cdr3b_only_neg_assays_test-0": "tab",
    "tchard_pep_cdr3b_only_neg_assays_test-1": "tab",
    "tchard_pep_cdr3b_only_neg_assays_test-2": "tab",
    "tchard_pep_cdr3b_only_neg_assays_test-3": "tab",
    "tchard_pep_cdr3b_only_neg_assays_test-4": "tab",
    "tchard_pep_cdr3b_only_neg_assays_train-0": "tab",
    "tchard_pep_cdr3b_only_neg_assays_train-1": "tab",
    "tchard_pep_cdr3b_only_neg_assays_train-2": "tab",
    "tchard_pep_cdr3b_only_neg_assays_train-3": "tab",
    "tchard_pep_cdr3b_only_neg_assays_train-4": "tab",
    "tchard_pep_cdr3b_only_sampled_negs_test-0": "tab",
    "tchard_pep_cdr3b_only_sampled_negs_test-1": "tab",
    "tchard_pep_cdr3b_only_sampled_negs_test-2": "tab",
    "tchard_pep_cdr3b_only_sampled_negs_test-3": "tab",
    "tchard_pep_cdr3b_only_sampled_negs_test-4": "tab",
    "tchard_pep_cdr3b_only_sampled_negs_train-0": "tab",
    "tchard_pep_cdr3b_only_sampled_negs_train-1": "tab",
    "tchard_pep_cdr3b_only_sampled_negs_train-2": "tab",
    "tchard_pep_cdr3b_only_sampled_negs_train-3": "tab",
    "tchard_pep_cdr3b_only_sampled_negs_train-4": "tab",
    "cell_tissue_mg_edgelist": "txt",
    "pinnacle_global_ppi_edgelist": "txt",
    "pinnacle_protein_embed": "pth",
    "pinnacle_labels_dict": "txt",
    "panpep": "tab",
    "pinnacle_output1": "zip",
    "pinnacle_output2": "zip",
    "pinnacle_output3": "zip",
    "pinnacle_output4": "zip",
    "pinnacle_output5": "zip",
    "pinnacle_output6": "zip",
    "pinnacle_output7": "zip",
    "pinnacle_output8": "zip",
    "pinnacle_output9": "zip",
    "pinnacle_output10": "zip",
    "geneformer_gene_median_dictionary": "pkl",
    "geneformer_gene_name_id_dict": "pkl",
    "geneformer_token_dictionary": "pkl",
    "evebio_pharmone_v1_assay_doc": "tab",
    "evebio_pharmone_v1_assay_table": "tab",
    "evebio_pharmone_v1_bundle_doc": "tab",
    "evebio_pharmone_v1_bundle_table": "tab",
    "evebio_pharmone_v1_compound_doc": "tab",
    "evebio_pharmone_v1_compound_table": "tab",
    "evebio_pharmone_v1_control_doc": "tab",
    "evebio_pharmone_v1_control_table": "tab",
    "evebio_pharmone_v1_detailed_result_doc": "tab",
    "evebio_pharmone_v1_detailed_result_table": "tab",
    "evebio_pharmone_v1_observed_points_doc": "tab",
    "evebio_pharmone_v1_observed_points_table": "tab",
    "evebio_pharmone_v1_summary_result_doc": "tab",
    "evebio_pharmone_v1_summary_result_table": "tab",
    "evebio_pharmone_v1_target_doc": "tab",
    "evebio_pharmone_v1_target_table": "tab",
    "cellxgene_sample_small": "h5ad",
    "scgpt_vocab": "json",
}

name2id = {
    "bbb_adenot": 4259565,
    "bbb_martins": 4259566,
    "b3db_classification": 7878566,
    "b3db_regression": 7878567,
    "bindingdb_ic50": 4291560,
    "bindingdb_kd": 4291555,
    "bindingdb_ki": 4291556,
    "bindingdb_patent": 4724851,
    "bioavailability_ma": 4259567,
    "caco2_wang": 4259569,
    "pampa_ncats": 6695858,
    "approved_pampa_ncats": 6695857,
    "clearance_edrug3d": 4259571,
    "clintox": 4259572,
    "cyp1a2_veith": 4259573,
    "cyp2c19_veith": 4259576,
    "cyp2c9_veith": 4259577,
    "cyp2d6_veith": 4259580,
    "cyp3a4_veith": 4259582,
    "cyp2c9_substrate_carbonmangels": 4259584,
    "cyp2d6_substrate_carbonmangels": 4259578,
    "cyp3a4_substrate_carbonmangels": 4259581,
    "carcinogens_lagunin": 4259570,
    "davis": 5219748,
    "drugbank": 4139573,
    "drugcomb": 4215720,
    "f20_edrug3d": 4259586,
    "f30_edrug3d": 4259589,
    "halflife_edrug3d": 4259587,
    "hia_hou": 4259591,
    "hiv": 4259593,
    "huri": 4139567,
    "hydrationfreeenergy_freesolv": 4259594,
    "kiba": 5255037,
    "lipophilicity_astrazeneca": 4259595,
    "pgp_broccatelli": 4259597,
    "ppbr_edrug3d": 4259600,
    "ppbr_ma": 4259603,
    "sarscov2_3clpro_diamond": 4259606,
    "sarscov2_vitro_touret": 4259607,
    "orexin1_receptor_butkiewicz": 6894447,
    "m1_muscarinic_receptor_agonists_butkiewicz": 6894443,
    "m1_muscarinic_receptor_antagonists_butkiewicz": 6894446,
    "potassium_ion_channel_kir2.1_butkiewicz": 6894442,
    "kcnq2_potassium_channel_butkiewicz": 6894444,
    "cav3_t-type_calcium_channels_butkiewicz": 6894445,
    "choline_transporter_butkiewicz": 6894441,
    "serine_threonine_kinase_33_butkiewicz": 6894448,
    "tyrosyl-dna_phosphodiesterase_butkiewicz": 6894440,
    "solubility_aqsoldb": 4259610,
    "tox21": 4259612,
    "toxcast": 4259613,
    "twosides": 4139574,
    "vd_edrug3d": 4259618,
    "mhc1_iedb-imgt_nielsen": 4167073,
    "mhc2_iedb_jensen": 4167074,
    "zinc": 4170963,
    "moses": 4170962,
    "chembl": 4170965,
    "chembl_v29": 5767979,
    "qed": 4170959,
    "drd2": 4170957,
    "logp": 4170961,
    "gdsc1": 4165726,
    "gdsc2": 4165727,
    "iedb_jespersen": 4165725,
    "pdb_jespersen": 4165724,
    "qm7": 6358510,
    "qm7b": 6358512,
    "qm8": 6358513,
    "qm9": 6179310,  ### 4167112, 6175612
    #  'scpdb': None,
    #  'dude': None,
    #  'crossdock': None,
    "tap": 4167113,
    "sabdab_chen": 4167164,
    "protein_sabdab": 4167357,
    "oncopolypharmacology": 4167358,
    "mirtarbase": 4167359,
    "disgenet": 4168282,
    "sabdab_liberis": 4168425,
    "uspto50k": 4171823,
    "buchwald-hartwig": 6175640,
    "uspto_yields": 4186956,
    "uspto_catalyst": 4171574,
    "uspto": 4171642,
    "hetionet": 4201734,
    "herg": 4259588,
    "herg_central": 5740618,
    "herg_karim": 6822246,
    "dili": 4259585,
    "ppbr_az": 6413140,
    "ames": 4259564,
    "skin_reaction": 4259609,
    "clearance_microsome_az": 4266186,
    "clearance_hepatocyte_az": 4266187,
    "ld50_zhu": 4267146,
    "half_life_obach": 4266799,
    "vdss_lombardo": 4267387,
    "leenay": 4279966,
    "test_single_pred": 4832455,
    "test_multi_pred": 4832456,
    "gdsc_gene_symbols": 5255026,
    "weber": 5790963,
    "primekg": 6180626,
    "primekg_drug_feature": 6180619,
    "primekg_disease_feature": 6180618,
    "drug_comb_meta_data": 7104245,
    "phase1": 7331305,
    "phase2": 7331306,
    "phase3": 7331307,
    "brown_mdm2_ace2_12ca5": 9649623,
    "scperturb_drug_AissaBenevolenskaya2021": 9845396,
    "scperturb_drug_SrivatsanTrapnell2020_sciplex2": 9845394,
    "scperturb_drug_SrivatsanTrapnell2020_sciplex3": 9845397,
    "scperturb_drug_SrivatsanTrapnell2020_sciplex4": 9845395,
    "scperturb_drug_ZhaoSims2021": 9845393,
    "scperturb_gene_NormanWeissman2019": 10133995,
    "scperturb_gene_ReplogleWeissman2022_rpe1": 10133996,
    "scperturb_gene_ReplogleWeissman2022_k562_essential": 10134031,
    "opentargets_ra_data_splits": 10141152,
    "opentargets_ibd_data_splits": 10141151,
    "opentargets_ra_data_splits_idx": 10143574,
    "opentargets_ibd_data_splits_idx": 10143573,
    "opentargets_ra_drug_evidence": 10141153,
    "opentargets_ibd_drug_evidence": 10141154,
    "hlm": 10218426,
    "rlm": 10218425,
    "tchard_full": 10228321,
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-0": 10228304,
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-1": 10228296,
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-2": 10228328,
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-3": 10228299,
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_test-4": 10228330,
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-0": 10228331,
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-1": 10228334,
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-2": 10228324,
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-3": 10228325,
    "tchard_pep_cdr3b_cdr3a_mhc_only_neg_assays_train-4": 10228327,
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-0": 10228320,
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-1": 10228295,
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-2": 10228297,
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-3": 10228294,
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_test-4": 10228309,
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-0": 10228301,
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-1": 10228310,
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-2": 10228315,
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-3": 10228311,
    "tchard_pep_cdr3b_cdr3a_mhc_only_sampled_negs_train-4": 10228335,
    "tchard_pep_cdr3b_only_neg_assays_test-0": 10228300,
    "tchard_pep_cdr3b_only_neg_assays_test-1": 10228302,
    "tchard_pep_cdr3b_only_neg_assays_test-2": 10228305,
    "tchard_pep_cdr3b_only_neg_assays_test-3": 10228298,
    "tchard_pep_cdr3b_only_neg_assays_test-4": 10228319,
    "tchard_pep_cdr3b_only_neg_assays_train-0": 10228312,
    "tchard_pep_cdr3b_only_neg_assays_train-1": 10228317,
    "tchard_pep_cdr3b_only_neg_assays_train-2": 10228333,
    "tchard_pep_cdr3b_only_neg_assays_train-3": 10228318,
    "tchard_pep_cdr3b_only_neg_assays_train-4": 10228314,
    "tchard_pep_cdr3b_only_sampled_negs_test-0": 10228329,
    "tchard_pep_cdr3b_only_sampled_negs_test-1": 10228332,
    "tchard_pep_cdr3b_only_sampled_negs_test-2": 10228303,
    "tchard_pep_cdr3b_only_sampled_negs_test-3": 10228306,
    "tchard_pep_cdr3b_only_sampled_negs_test-4": 10228308,
    "tchard_pep_cdr3b_only_sampled_negs_train-0": 10228323,
    "tchard_pep_cdr3b_only_sampled_negs_train-1": 10228313,
    "tchard_pep_cdr3b_only_sampled_negs_train-2": 10228322,
    "tchard_pep_cdr3b_only_sampled_negs_train-3": 10228316,
    "tchard_pep_cdr3b_only_sampled_negs_train-4": 10228326,
    "cell_tissue_mg_edgelist": 10407107,
    "pinnacle_global_ppi_edgelist": 10407108,
    "pinnacle_protein_embed": 10407128,
    "pinnacle_labels_dict": 10409635,
    "panpep": 10428565,
    "pinnacle_output1": 10431072,
    "pinnacle_output2": 10431073,
    "pinnacle_output3": 10431078,
    "pinnacle_output4": 10431080,
    "pinnacle_output5": 10431077,
    "pinnacle_output6": 10431076,
    "pinnacle_output7": 10431079,
    "pinnacle_output8": 10431074,
    "pinnacle_output9": 10431075,
    "pinnacle_output10": 10431081,
    "geneformer_gene_median_dictionary": 10626278,
    "geneformer_gene_name_id_dict": 10626276,
    "geneformer_token_dictionary": 10626277,
    "evebio_pharmone_v1_assay_doc": 10741530,
    "evebio_pharmone_v1_assay_table": 10741541,
    "evebio_pharmone_v1_bundle_doc": 10741540,
    "evebio_pharmone_v1_bundle_table": 10741543,
    "evebio_pharmone_v1_compound_doc": 10741535,
    "evebio_pharmone_v1_compound_table": 10741532,
    "evebio_pharmone_v1_control_doc": 10741539,
    "evebio_pharmone_v1_control_table": 10741538,
    "evebio_pharmone_v1_detailed_result_doc": 10741545,
    "evebio_pharmone_v1_detailed_result_table": 10741534,
    "evebio_pharmone_v1_observed_points_doc": 10741531,
    "evebio_pharmone_v1_observed_points_table": 10741544,
    "evebio_pharmone_v1_summary_result_doc": 10741529,
    "evebio_pharmone_v1_summary_result_table": 10741542,
    "evebio_pharmone_v1_target_doc": 10741536,
    "evebio_pharmone_v1_target_table": 10741537,
    "cellxgene_sample_small": 10806522,
    "scgpt_vocab": 10809431,
}

oracle2type = {
    "drd2": "pkl",
    "jnk3": "pkl",
    "gsk3b": "pkl",
    "fpscores": "pkl",
    "cyp3a4_veith": "pkl",
    "smina": "static",
    "drd2_current": "pkl",
    "jnk3_current": "pkl",
    "gsk3b_current": "pkl",
}

oracle2id = {
    "drd2": 4178625,
    "gsk3b": 4170295,
    "jnk3": 4170293,
    "fpscores": 4170416,
    "cyp3a4_veith": 4411249,
    "smina": 6361665,
    "cyp3a4_veith": 4411249,
    "drd2_current": 6413411,
    "jnk3_current": 6413420,
    "gsk3b_current": 6413412,
}

benchmark2type = {
    "admet_group": "zip",
    "drugcombo_group": "zip",
    "docking_group": "zip",
    "dti_dg_group": "zip",
}

benchmark2id = {
    "admet_group": 4426004,
    "drugcombo_group": 4426002,
    "docking_group": 4554082,
    "dti_dg_group": 4742443,
}

receptor2id = {
    "1iep": [5137914, 5617659],
    "2rgp": [5137916, 5617662],
    "3eml": [5137919, 5617663],
    "3ny8": [5137915, 5617665],
    "4rlu": [5137918, 5617658],
    "4unn": [5137917, 5617661],
    "5mo4": [5137920, 5617664],
    "7l11": [5137921, 5617660],
    "3pbl": [5257195, 5617666],
}  ## 'drd3': 5137901,

sdf_file_names = {"grambow": ["Product", "Reactant", "TS"]}

name2stats = {
    "caco2_wang": 906,
    "hia_hou": 578,
    "pgp_broccatelli": 1212,
    "bioavailability_ma": 640,
    "lipophilicity_astrazeneca": 4200,
    "solubility_aqsoldb": 9982,
    "bbb_martins": 1975,
    "ppbr_az": 1797,
    "vdss_lombardo": 1130,
    "cyp2c19_veith": 12092,
    "cyp2d6_veith": 13130,
    "cyp3a4_veith": 12328,
    "cyp1a2_veith": 12579,
    "cyp2c9_veith": 12092,
    "cyp2c9_substrate_carbonmangels": 666,
    "cyp2d6_substrate_carbonmangels": 664,
    "cyp3a4_substrate_carbonmangels": 667,
    "half_life_obach": 667,
    "clearance_hepatocyte_az": 1020,
    "clearance_microsome_az": 1102,
    "ld50_zhu": 7385,
    "herg": 648,
    "ames": 7255,
    "dili": 475,
    "skin_reaction": 404,
    "carcinogens_lagunin": 278,
    "tox21": 7831,
    "clintox": 1484,
    "sarscov2_vitro_touret": 1480,
    "sarscov2_3clpro_diamond": 879,
    "hiv": 41127,
    "qm7": 7165,
    "qm7b": 7211,
    "qm8": 21747,
    "qm9": 133885,
    "uspto_yields": 853638,
    "buchwald-hartwig": 55370,
    "sabdab_liberis": 1023,
    "iedb_jespersen": 3159,
    "pdb_jespersen": 447,
    "tap": 242,
    "sabdab_chen": 2409,
    "leenay": 1521,
    "bindingdb_kd": 52284,
    "bindingdb_ki": 375032,
    "bindingdb_ic50": 991486,
    "bindingdb_patent": 243344,
    "davis": 27621,
    "kiba": 118036,
    "drugbank": 191808,
    "twosides": 4649441,
    "huri": 51813,
    "disgenet": 52476,
    "gdsc1": 177310,
    "gdsc2": 92703,
    "drugcomb": 297098,
    "oncopolypharmacology": 23052,
    "mhc1_iedb-imgt_nielsen": 185985,
    "mhc2_iedb_jensen": 134281,
    "protein_sabdab": 493,
    "mirtarbase": 400082,
    "uspto_catalyst": 721799,
    "moses": 1936962,
    "zinc": 249455,
    "chembl": 1961462,
    "uspto50k": 50036,
    "uspto": 1939253,
    "phase1": 1787,
    "phase2": 6102,
    "phase3": 4576,
}

name2idlist = {
    "dude": [6429245, 6429251],
    "scpdb": [6431629, 6431631],
}



# -*- coding: utf-8 -*-
# Author: TDC Team
# License: MIT

from .base_group import BenchmarkGroup


class dti_dg_group(BenchmarkGroup):
    """Create a DTI domain generalization benchmark group

    Args:
        path (str, optional): path to save/load benchmarks
    """

    def __init__(self, path="./data"):
        """Create a DTI domain generalization benchmark group"""
        super().__init__(name="DTI_DG_Group", path=path, file_format="csv")



from .brown_mdm2_ace2_12ca5_config import BrownProteinPeptideConfig
from .opentargets_dti import OpentargetsDTI
from .scperturb_config import SCPerturb, SCPerturb_Gene

scperturb_datasets = [
    "scperturb_drug_AissaBenevolenskaya2021",
    "scperturb_drug_SrivatsanTrapnell2020_sciplex2",
    "scperturb_drug_SrivatsanTrapnell2020_sciplex3",
    "scperturb_drug_SrivatsanTrapnell2020_sciplex4",
    "scperturb_drug_ZhaoSims2021",
]

scperturb_gene_datasets = [
    "scperturb_gene_NormanWeissman2019",
    "scperturb_gene_ReplogleWeissman2022_rpe1",
    "scperturb_gene_ReplogleWeissman2022_k562_essential",
]


class ConfigMap(dict):
    """
    The ConfigMap stores key-value pairs where the key is a dataset string name and the value is a config class.
    """

    def __init__(self):
        self["brown_mdm2_ace2_12ca5"] = BrownProteinPeptideConfig
        for ds in scperturb_datasets:
            self[ds] = SCPerturb
        for ds in scperturb_gene_datasets:
            self[ds] = SCPerturb_Gene
        self["opentargets_dti"] = OpentargetsDTI



# -*- coding: utf-8 -*-
# Author: TDC Team
# License: MIT

import numpy as np

try:
    from rdkit import rdBase

    rdBase.DisableLog("rdApp.error")
except:
    raise ImportError(
        "Please install rdkit by 'conda install -c conda-forge rdkit'! ")

from ...utils import print_sys, install


class MolFilter:
    """Molecule Filter: filter Molecule based on user-specified condition

    Args:
      filters:
      property_filters_flag: bool,
      HBA: [lower_bound, upper_bound]
      HBD: [lower_bound, upper_bound]
      LogP: [lower_bound, upper_bound]
      MW: [lower_bound, upper_bound],   Molecule weight
      Rot: [lower_bound, upper_bound]
      TPSA: [lower_bound, upper_bound]

    Returns:
      list of SMILES strings that pass the filter.

    """

    # MIT License: Checkout https://github.com/PatWalters/rd_filters
    def __init__(
        self,
        filters="all",
        property_filters_flag=True,
        HBA=[0, 10],
        HBD=[0, 5],
        LogP=[-5, 5],
        MW=[0, 500],
        Rot=[0, 10],
        TPSA=[0, 200],
    ):
        try:
            from rd_filters.rd_filters import RDFilters, read_rules
        except:
            install("git+https://github.com/PatWalters/rd_filters.git")
            from rd_filters.rd_filters import RDFilters, read_rules

        import pkg_resources

        self.property_filters_flag = property_filters_flag
        all_filters = [
            "BMS",
            "Dundee",
            "Glaxo",
            "Inpharmatica",
            "LINT",
            "MLSMR",
            "PAINS",
            "SureChEMBL",
        ]
        if filters == "all":
            filters = all_filters
        else:
            if isinstance(filters, str):
                filters = [filters]
            if isinstance(filters, list):
                ## a set of filters
                for i in filters:
                    if i not in all_filters:
                        raise ValueError(
                            i +
                            " not found; Please choose from a list of available filters from 'BMS', 'Dundee', 'Glaxo', 'Inpharmatica', 'LINT', 'MLSMR', 'PAINS', 'SureChEMBL'"
                        )

        alert_file_name = pkg_resources.resource_filename(
            "rd_filters", "data/alert_collection.csv")
        rules_file_path = pkg_resources.resource_filename(
            "rd_filters", "data/rules.json")
        self.rf = RDFilters(alert_file_name)
        self.rule_dict = read_rules(rules_file_path)
        self.rule_dict["Rule_Inpharmatica"] = False
        for i in filters:
            self.rule_dict["Rule_" + i] = True

        if self.property_filters_flag:
            (
                self.rule_dict["HBA"],
                self.rule_dict["HBD"],
                self.rule_dict["LogP"],
                self.rule_dict["MW"],
                self.rule_dict["Rot"],
                self.rule_dict["TPSA"],
            ) = (HBA, HBD, LogP, MW, Rot, TPSA)
        else:
            if "HBA" in self.rule_dict:
                del self.rule_dict["HBA"]
            if "HBD" in self.rule_dict:
                del self.rule_dict["HBD"]
            if "LogP" in self.rule_dict:
                del self.rule_dict["LogP"]
            if "MW" in self.rule_dict:
                del self.rule_dict["MW"]
            if "Rot" in self.rule_dict:
                del self.rule_dict["Rot"]
            if "TPSA" in self.rule_dict:
                del self.rule_dict["TPSA"]
            # del self.rule_dict['HBA'], self.rule_dict['HBD'], self.rule_dict['LogP'], self.rule_dict['MW'], self.rule_dict['Rot'], self.rule_dict['TPSA']
        print_sys("MolFilter is using the following filters:")

        for i, j in self.rule_dict.items():
            if i[:4] == "Rule":
                if j:
                    print_sys(i + ": " + str(j))
            else:
                print_sys(i + ": " + str(j))
        rule_list = [
            x.replace("Rule_", "")
            for x in self.rule_dict.keys()
            if x.startswith("Rule") and self.rule_dict[x]
        ]
        rule_str = " and ".join(rule_list)
        self.rf.build_rule_list(rule_list)

    def __call__(self, input_data):
        import multiprocessing as mp
        from multiprocessing import Pool
        import pandas as pd

        if isinstance(input_data, str):
            input_data = [input_data]
        elif not isinstance(input_data, (list, np.ndarray, np.generic)):
            raise ValueError(
                "Input must be a list/numpy array of SMILES or one SMILES string!"
            )

        input_data = list(tuple(zip(input_data, list(range(len(input_data))))))

        num_cores = int(mp.cpu_count())
        p = Pool(num_cores)

        res = list(p.map(self.rf.evaluate, input_data))

        if self.property_filters_flag:

            df = pd.DataFrame(
                res,
                columns=[
                    "SMILES",
                    "NAME",
                    "FILTER",
                    "MW",
                    "LogP",
                    "HBD",
                    "HBA",
                    "TPSA",
                    "Rot",
                ],
            )
            df_ok = df[(df.FILTER == "OK") &
                       df.MW.between(*self.rule_dict["MW"]) &
                       df.LogP.between(*self.rule_dict["LogP"]) &
                       df.HBD.between(*self.rule_dict["HBD"]) &
                       df.HBA.between(*self.rule_dict["HBA"]) &
                       df.TPSA.between(*self.rule_dict["TPSA"]) &
                       df.Rot.between(*self.rule_dict["Rot"])]

        else:
            df = pd.DataFrame(
                res,
                columns=[
                    "SMILES",
                    "NAME",
                    "FILTER",
                    "MW",
                    "LogP",
                    "HBD",
                    "HBA",
                    "TPSA",
                    "Rot",
                ],
            )
            df_ok = df[(df.FILTER == "OK")]
        return df_ok.SMILES.values



# -*- coding: utf-8 -*-
# Author: TDC Team
# License: MIT

import warnings

warnings.filterwarnings("ignore")
import sys

from ..utils import print_sys
from . import bi_pred_dataset, multi_pred_dataset
from ..metadata import dataset_names


class AntibodyAff(bi_pred_dataset.DataLoader):
    """Data loader class to load datasets in Antibody-antigen Affinity Prediction task.
    More info: https://tdcommons.ai/multi_pred_tasks/antibodyaff/

    Task Description: Regression. Given the amino acid sequence of antibody and antigen, predict their binding affinity.


    Args:
        name (str): the dataset name.
        path (str, optional):
            The path to save the data file, defaults to './data'
        label_name (str, optional):
            For multi-label dataset, specify the label name, defaults to None
        print_stats (bool, optional):
            Whether to print basic statistics of the dataset, defaults to False


    """

    def __init__(self, name, path="./data", label_name=None, print_stats=False):
        """Create Antibody-antigen Affinity dataloader object"""
        super().__init__(
            name,
            path,
            label_name,
            print_stats,
            dataset_names=dataset_names["AntibodyAff"],
        )
        self.entity1_name = "Antibody"
        self.entity2_name = "Antigen"
        self.two_types = True

        if print_stats:
            self.print_stats()

        print("Done!", flush=True, file=sys.stderr)



# -*- coding: utf-8 -*-
# Author: TDC Team
# License: MIT

import numpy as np
import os

try:
    from rdkit import Chem, DataStructs
    from rdkit.Chem import AllChem
    from rdkit import rdBase

    rdBase.DisableLog("rdApp.error")
except:
    raise ImportError(
        "Please install rdkit by 'conda install -c conda-forge rdkit'! ")


def single_molecule_validity(smiles):
    """Evaluate the chemical validity of a single molecule in terms of SMILES string

    Args:
      smiles: str, SMILES string.

    Returns:
      Boolean: if the SMILES string is a valid molecule

    """
    if smiles.strip() == "":
        return False
    mol = Chem.MolFromSmiles(smiles)
    if mol is None or mol.GetNumAtoms() == 0:
        return False
    return True


def validity(list_of_smiles):
    valid_list_smiles = list(filter(single_molecule_validity, list_of_smiles))
    return 1.0 * len(valid_list_smiles) / len(list_of_smiles)


def canonicalize(smiles):
    """Convert SMILES into canonical form.

    Args:
      smiles: str, SMILES string

    Returns:
      smiles: str, canonical SMILES string.

    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is not None:
        return Chem.MolToSmiles(mol, isomericSmiles=True)
    else:
        return None


def unique_lst_of_smiles(list_of_smiles):
    canonical_smiles_lst = list(map(canonicalize, list_of_smiles))
    canonical_smiles_lst = list(
        filter(lambda x: x is not None, canonical_smiles_lst))
    canonical_smiles_lst = list(set(canonical_smiles_lst))
    return canonical_smiles_lst


def uniqueness(list_of_smiles):
    """Evaluate the uniqueness of a list of SMILES string, i.e., the fraction of unique molecules among a given list.

    Args:
      list_of_smiles: list (of SMILES string)

    Returns:
      uniqueness: float
    """
    canonical_smiles_lst = unique_lst_of_smiles(list_of_smiles)
    return 1.0 * len(canonical_smiles_lst) / len(list_of_smiles)


def novelty(generated_smiles_lst, training_smiles_lst):
    """Evaluate the novelty of set of generated smiles using list of training smiles as reference.
    Novelty is defined as the fraction of generated molecules that doesn't appear in the training set.

    Args:
      generated_smiles_lst: list (of SMILES string), which are generated.
      training_smiles_lst: list (of SMILES string), which are used for training.

    Returns:
      novelty: float
    """
    generated_smiles_lst = unique_lst_of_smiles(generated_smiles_lst)
    training_smiles_lst = unique_lst_of_smiles(training_smiles_lst)
    novel_ratio = (sum(
        [1 if i in training_smiles_lst else 0 for i in generated_smiles_lst]) *
                   1.0 / len(generated_smiles_lst))
    return 1 - novel_ratio


def diversity(list_of_smiles):
    """Evaluate the internal diversity of a set of molecules. The internbal diversity is defined as the average pairwise
      Tanimoto distance between the Morgan fingerprints.

    Args:
      list_of_smiles: list of SMILES strings

    Returns:
      div: float
    """
    list_of_unique_smiles = unique_lst_of_smiles(list_of_smiles)
    list_of_mol = [
        Chem.MolFromSmiles(smiles) for smiles in list_of_unique_smiles
    ]
    list_of_fp = [
        AllChem.GetMorganFingerprintAsBitVect(mol,
                                              2,
                                              nBits=2048,
                                              useChirality=False)
        for mol in list_of_mol
    ]
    avg_lst = []
    for idx, fp in enumerate(list_of_fp):
        for fp2 in list_of_fp[idx + 1:]:
            sim = DataStructs.TanimotoSimilarity(fp, fp2)
            ### option I
            distance = 1 - sim
            ### option II
            # distance = -np.log2(sim)
            avg_lst.append(distance)
    div = np.mean(avg_lst)
    return div


########################################
######## KL divergence ########


def _calculate_pc_descriptors(smiles, pc_descriptors):
    """Calculate Physical Chemical descriptors of a single SMILES (internal function).

    Args:
      list_of_smiles: SMILES strings
      pc_descriptors: list of strings, names of descriptors to calculate

    Returns:
      descriptros: list of float
    """
    from rdkit.ML.Descriptors import MoleculeDescriptors

    calc = MoleculeDescriptors.MolecularDescriptorCalculator(pc_descriptors)

    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None
    _fp = calc.CalcDescriptors(mol)
    _fp = np.array(_fp)
    mask = np.isfinite(_fp)
    if (mask == 0).sum() > 0:
        logger.warning(f"{smiles} contains an NAN physchem descriptor")
        _fp[~mask] = 0

    return _fp


def calculate_pc_descriptors(smiles, pc_descriptors):
    """Calculate Physical Chemical descriptors of a list of molecules.

    Args:
      list_of_smiles: list of SMILES strings
      pc_descriptors: list of strings, names of descriptors to calculate

    Returns:
      descriptros: list of float
    """
    output = []

    for i in smiles:
        d = _calculate_pc_descriptors(i, pc_descriptors)
        if d is not None:
            output.append(d)

    return np.array(output)


def continuous_kldiv(X_baseline: np.array, X_sampled: np.array) -> float:
    """calculate KL divergence for two numpy arrays, conitnuous version.

    Args:
      X_baseline: numpy array
      X_sampled: numpy array

    Returns:
      KL divergence: float
    """
    X_baseline += 1e-5
    X_sampled += 1e-5
    from scipy.stats import entropy, gaussian_kde

    kde_P = gaussian_kde(X_baseline)
    kde_Q = gaussian_kde(X_sampled)
    x_eval = np.linspace(
        np.hstack([X_baseline, X_sampled]).min(),
        np.hstack([X_baseline, X_sampled]).max(),
        num=1000,
    )
    P = kde_P(x_eval) + 1e-10
    Q = kde_Q(x_eval) + 1e-10

    return entropy(P, Q)


def discrete_kldiv(X_baseline: np.array, X_sampled: np.array) -> float:
    """calculate KL divergence for two numpy arrays, discrete version.

    Args:
      X_baseline: numpy array
      X_sampled: numpy array

    Returns:
      KL divergence: float
    """
    from scipy.stats import entropy
    from scipy import histogram

    P, bins = histogram(X_baseline, bins=10, density=True)
    P += 1e-10
    Q, _ = histogram(X_sampled, bins=bins, density=True)
    Q += 1e-10

    return entropy(P, Q)


def get_fingerprints(mols, radius=2, length=4096):
    """
    Converts molecules to ECFP bitvectors.

    Args:
        mols: RDKit molecules
        radius: ECFP fingerprint radius
        length: number of bits

    Returns: a list of fingerprints
    """
    return [
        AllChem.GetMorganFingerprintAsBitVect(m, radius, length) for m in mols
    ]


def get_mols(smiles_list):
    """Convert SMILES strings to RDKit RDMol objects.

    Args:
      list_of_smiles: list of SMILES strings

    Returns:
      mols: list of RDKit RDMol objects
    """
    for i in smiles_list:
        try:
            mol = Chem.MolFromSmiles(i)
            if mol is not None:
                yield mol
        except Exception as e:
            logger.warning(e)


def calculate_internal_pairwise_similarities(smiles_list):
    """
    Computes the pairwise similarities of the provided list of smiles against itself.

    Args:
        smiles_list: list of str

    Returns:
        Symmetric matrix of pairwise similarities. Diagonal is set to zero.
    """
    if len(smiles_list) > 10000:
        logger.warning(f"Calculating internal similarity on large set of "
                       f"SMILES strings ({len(smiles_list)})")

    mols = get_mols(smiles_list)
    fps = get_fingerprints(mols)
    nfps = len(fps)

    similarities = np.zeros((nfps, nfps))

    for i in range(1, nfps):
        sims = DataStructs.BulkTanimotoSimilarity(fps[i], fps[:i])
        similarities[i, :i] = sims
        similarities[:i, i] = sims

    return similarities


def kl_divergence(generated_smiles_lst, training_smiles_lst):
    """Evaluate the KL divergence of set of generated smiles using list of training smiles as reference.
    KL divergence is defined as the averaged KL divergence of a set of physical chemical descriptors
    between a set of generated molecules and a set of training molecules.

    Args:
      generated_smiles_lst: list (of SMILES string), which are generated.
      training_smiles_lst: list (of SMILES string), which are used for training.

    Returns:
      KL divergence: float
    """
    pc_descriptor_subset = [
        "BertzCT",
        "MolLogP",
        "MolWt",
        "TPSA",
        "NumHAcceptors",
        "NumHDonors",
        "NumRotatableBonds",
        "NumAliphaticRings",
        "NumAromaticRings",
    ]

    def canonical(smiles):
        mol = Chem.MolFromSmiles(smiles)
        if mol is not None:
            return Chem.MolToSmiles(mol,
                                    isomericSmiles=True)  ### todo double check
        else:
            return None

    generated_lst_mol = list(map(canonical, generated_smiles_lst))
    training_lst_mol = list(map(canonical, training_smiles_lst))
    filter_out_func = lambda x: x is not None
    generated_lst_mol = list(filter(filter_out_func, generated_lst_mol))
    training_lst_mol = list(filter(filter_out_func, training_lst_mol))

    d_sampled = calculate_pc_descriptors(generated_lst_mol,
                                         pc_descriptor_subset)
    d_chembl = calculate_pc_descriptors(training_lst_mol, pc_descriptor_subset)

    kldivs = {}
    for i in range(4):
        kldiv = continuous_kldiv(X_baseline=d_chembl[:, i],
                                 X_sampled=d_sampled[:, i])
        kldivs[pc_descriptor_subset[i]] = kldiv

    # ... and for the int valued ones.
    for i in range(4, 9):
        kldiv = discrete_kldiv(X_baseline=d_chembl[:, i],
                               X_sampled=d_sampled[:, i])
        kldivs[pc_descriptor_subset[i]] = kldiv

    # pairwise similarity

    chembl_sim = calculate_internal_pairwise_similarities(training_lst_mol)
    chembl_sim = chembl_sim.max(axis=1)

    sampled_sim = calculate_internal_pairwise_similarities(generated_lst_mol)
    sampled_sim = sampled_sim.max(axis=1)

    kldiv_int_int = continuous_kldiv(X_baseline=chembl_sim,
                                     X_sampled=sampled_sim)
    kldivs["internal_similarity"] = kldiv_int_int
    """
        # for some reason, this runs into problems when both sets are identical.
        # cross_set_sim = calculate_pairwise_similarities(self.training_set_molecules, unique_molecules)
        # cross_set_sim = cross_set_sim.max(axis=1)
        #
        # kldiv_ext = discrete_kldiv(chembl_sim, cross_set_sim)
        # kldivs['external_similarity'] = kldiv_ext
        # kldiv_sum += kldiv_ext
  """

    # Each KL divergence value is transformed to be in [0, 1].
    # Then their average delivers the final score.
    partial_scores = [np.exp(-score) for score in kldivs.values()]
    score = sum(partial_scores) / len(partial_scores)
    return score


def fcd_distance_tf(generated_smiles_lst, training_smiles_lst):
    """Evaluate FCD distance between generated smiles set and training smiles set using tensorflow.

    Args:
      generated_smiles_lst: list (of SMILES string), which are generated.
      training_smiles_lst: list (of SMILES string), which are used for training.

    Returns:
      fcd_distance: float
    """
    import pkgutil, tempfile, os

    if "chemnet" not in globals().keys():
        global chemnet
        ### _load_chemnet
        chemnet_model_filename = "ChemNet_v0.13_pretrained.h5"
        model_bytes = pkgutil.get_data("fcd", chemnet_model_filename)
        tmpdir = tempfile.gettempdir()
        model_path = os.path.join(tmpdir, chemnet_model_filename)
        with open(model_path, "wb") as f:
            f.write(model_bytes)
        chemnet = fcd.load_ref_model(model_path)
        # _load_chemnet

    def _calculate_distribution_statistics(chemnet, molecules):
        sample_std = fcd.canonical_smiles(molecules)
        gen_mol_act = fcd.get_predictions(chemnet, sample_std)

        mu = np.mean(gen_mol_act, axis=0)
        cov = np.cov(gen_mol_act.T)
        return mu, cov

    mu_ref, cov_ref = _calculate_distribution_statistics(
        chemnet, training_smiles_lst)
    mu, cov = _calculate_distribution_statistics(chemnet, generated_smiles_lst)

    FCD = fcd.calculate_frechet_distance(mu1=mu_ref,
                                         mu2=mu,
                                         sigma1=cov_ref,
                                         sigma2=cov)
    fcd_distance = np.exp(-0.2 * FCD)
    return fcd_distance


def fcd_distance_torch(generated_smiles_lst, training_smiles_lst):
    """Evaluate FCD distance between generated smiles set and training smiles set using PyTorch.

    Args:
      generated_smiles_lst: list (of SMILES string), which are generated.
      training_smiles_lst: list (of SMILES string), which are used for training.

    Returns:
      fcd_distance: float
    """
    os.environ["KMP_DUPLICATE_LIB_OK"] = "True"
    from fcd_torch import FCD

    fcd = FCD(device="cpu", n_jobs=8)
    fcd_distance = fcd(generated_smiles_lst, training_smiles_lst)
    return fcd_distance


def fcd_distance(generated_smiles_lst, training_smiles_lst):
    """Evaluate FCD distance between generated smiles set and training smiles set.

    Args:
      generated_smiles_lst: list (of SMILES string), which are generated.
      training_smiles_lst: list (of SMILES string), which are used for training.

    Returns:
      fcd_distance: float
    """
    try:
        import tensorflow, fcd

        global fcd
    except:
        try:
            import torch, fcd_torch

            return fcd_distance_torch(generated_smiles_lst, training_smiles_lst)
        except:
            raise ImportError(
                "Please install fcd by 'pip install FCD' (for Tensorflow backend) \
                                            or 'pip install fcd_torch' (for PyTorch backend)!"
            )
    return fcd_distance_tf(generated_smiles_lst, training_smiles_lst)



# -*- coding: utf-8 -*-
# Author: TDC Team
# License: MIT
import os

from .base_group import BenchmarkGroup


class TCREpitopeGroup(BenchmarkGroup):
    """Create SCDTI Group Class object. This is for single-cell drug-target identification task benchmark.

    Args:
            path (str, optional): the path to store/retrieve the SCDTI group datasets.
    """

    def __init__(self, path="./data", file_format="csv"):
        """Create an SCDTI benchmark group class."""
        # super().__init__(name="SCDTI_Group", path=path)
        self.name = "TCREpitopeGroup"
        self.path = os.path.join(path, self.name)
        # self.datasets = ["opentargets_dti"]
        self.dataset_names = ["tchard"]
        self.file_format = file_format
        self.split = None

    def get_train_valid_split(self):
        """parameters included for compatibility. this benchmark has a fixed train/test split."""
        from ..resource.dataloader import DataLoader
        if self.split is None:
            dl = DataLoader(name="tchard")
            self.split = dl.get_split()
        return self.split["train"], self.split["dev"]

    def get_test(self):
        if self.split is None:
            self.get_train_valid_split()
        return self.split["test"]

    def evaluate(self, y_pred):
        import pandas as pd
        from sklearn.metrics import precision_score, recall_score, accuracy_score, f1_score, roc_auc_score
        y_true = self.get_test()
        aucs = []
        # Calculate metrics
        for neg_method, splits in y_true.items():
            for key, df in splits.items():
                assert type(df) == pd.DataFrame, (type(df), df)
                # compute metrics
                true = df["Y"]
                pred = y_pred[neg_method][key]["Y"]
                precision = precision_score(true, pred)
                recall = recall_score(true, pred)
                accuracy = accuracy_score(true, pred)
                f1 = f1_score(true, pred)
                auc = roc_auc_score(true, pred)
                y_pred[neg_method][key]["precision"] = precision
                y_pred[neg_method][key]["recall"] = recall
                y_pred[neg_method][key]["accuracy"] = accuracy
                y_pred[neg_method][key]["f1"] = f1
                y_pred[neg_method][key]["auc"] = auc
                aucs.append((auc, len(df)))
        total_samples = sum(x[1] for x in aucs)
        weighted_sum = sum(a * size for a, size in aucs)
        return weighted_sum / total_samples

    def evaluate_many(self, preds):
        from numpy import mean, std
        if len(preds) < 5:
            raise Exception(
                "Run your model on at least 5 seeds to compare results and provide your outputs in preds."
            )
        weighted_aucs = [self.evaluate(p) for p in preds]
        return {"mean_auc": mean(weighted_aucs), "std_aucs": std(weighted_aucs)}



# -*- coding: utf-8 -*-
# Author: TDC Team
# License: MIT
"""
This file contains a base data loader object that specific one can inherit from. 
"""

import pandas as pd
import numpy as np
import sys
import warnings

warnings.filterwarnings("ignore")

from . import utils


class DataLoader:
    """base data loader class that contains functions shared by almost all data loader classes."""

    def __init__(self):
        """empty data loader class, to be overwritten"""
        pass

    def get_data(self, format="df"):
        """
        Arguments:
            format (str, optional): the dataset format

        Returns:
            pd.DataFrame/dict/np.array: when format is df/dict/DeepPurpose

        Raises:
            AttributeError: format not supported
        """
        if format == "df":
            return pd.DataFrame({
                self.entity1_name + "_ID": self.entity1_idx,
                self.entity1_name: self.entity1,
                "Y": self.y,
            })
        elif format == "dict":
            return {
                self.entity1_name + "_ID": self.entity1_idx,
                self.entity1_name: self.entity1,
                "Y": self.y,
            }
        elif format == "DeepPurpose":
            return self.entity1, self.y
        else:
            raise AttributeError("Please use the correct format input")

    def print_stats(self):
        """print statistics"""
        print(
            "There are " + str(len(np.unique(self.entity1))) + " unique " +
            self.entity1_name.lower() + "s",
            flush=True,
            file=sys.stderr,
        )

    def get_split(self, method="random", seed=42, frac=[0.7, 0.1, 0.2]):
        """
        split function, overwritten by single_pred/multi_pred/generation for more specific splits

        Arguments:
            method: splitting schemes
            seed: random seed
            frac: train/val/test split fractions

        Returns:
            dict: a dictionary of train/valid/test dataframes

        Raises:
            AttributeError: split method not supported
        """

        df = self.get_data(format="df")

        if method == "random":
            return utils.create_fold(df, seed, frac)
        elif method == "cold_" + self.entity1_name.lower():
            return utils.create_fold_setting_cold(df, seed, frac,
                                                  self.entity1_name)
        else:
            raise AttributeError("Please specify the correct splitting method")

    def label_distribution(self):
        """visualize distribution of labels"""
        utils.label_dist(self.y, self.name)

    def binarize(self, threshold=None, order="descending"):
        """binarize the labels

        Args:
            threshold (float, optional): the threshold to binarize the label.
            order (str, optional): the order of binarization, if ascending, flip 1 to larger values and vice versus for descending

        Returns:
            DataLoader: data loader class with updated label

        Raises:
            AttributeError: no threshold specified for binarization
        """
        if threshold is None:
            raise AttributeError(
                "Please specify the threshold to binarize the data by "
                "'binarize(threshold = N)'!")

        if len(np.unique(self.y)) == 2:
            print("The data is already binarized!", flush=True, file=sys.stderr)
        else:
            print(
                "Binariztion using threshold " + str(threshold) +
                ", default, we assume the smaller values are 1 "
                "and larger ones is 0, you can change the order "
                "by 'binarize(order = 'ascending')'",
                flush=True,
                file=sys.stderr,
            )
            if (np.unique(self.y).reshape(-1,).shape[0] < 2):
                raise AttributeError(
                    "Adjust your threshold, there is only one class.")
            self.y = utils.binarize(self.y, threshold, order)
        return self

    def __len__(self):
        """get number of data points

        Returns:
            int: number of data points
        """
        return len(self.get_data(format="df"))

    def convert_to_log(self, form="standard"):
        """convert labels to log-scale

        Args:
            form (str, optional): standard log-transformation or binding nM <-> p transformation.
        """
        print("To log space...", flush=True, file=sys.stderr)
        self.log_flag = True
        if form == "binding":
            self.y = utils.convert_to_log(self.y)
        elif form == "standard":
            self.sign = np.sign(self.y)
            self.y = self.sign * np.log(abs(self.y) + 1e-10)

    def convert_from_log(self, form="standard"):
        """convert labels from log-scale

        Args:
            form (str, optional): standard log-transformation or binding nM <-> p transformation.
        """
        print("Convert Back To Original space...", flush=True, file=sys.stderr)
        if form == "binding":
            self.y = utils.convert_back_log(self.y)
        elif form == "standard":
            self.y = self.sign * (np.exp(self.sign * self.y) - 1e-10)
        self.log_flag = False

    def get_label_meaning(self, output_format="dict"):
        """get the biomedical meaning of label

        Args:
            output_format (str, optional): dict/df/array for label

        Returns:
            dict/pd.DataFrame/np.array: when output_format is dict/df/array
        """
        return utils.get_label_map(
            self.name,
            self.path,
            self.target,
            file_format=self.file_format,
            output_format=output_format,
        )

    def balanced(self, oversample=False, seed=42):
        """balance the label neg-pos ratio

        Args:
            oversample (bool, optional): whether or not to oversample minority or subsample majority to match ratio
            seed (int, optional): random seed

        Returns:
            pd.DataFrame: the updated dataframe with balanced dataset

        Raises:
            AttributeError: alert to binarize the data first as continuous values cannot do balancing
        """
        if len(np.unique(self.y)) > 2:
            raise AttributeError(
                "You should binarize the data first by calling "
                "data.binarize(threshold)",
                flush=True,
                file=sys.stderr,
            )

        val = self.get_data()

        class_ = val.Y.value_counts().keys().values
        major_class = class_[0]
        minor_class = class_[1]

        if not oversample:
            print(
                " Subsample the majority class is used, if you want to do "
                "oversample the minority class, set 'balanced(oversample = True)'. ",
                flush=True,
                file=sys.stderr,
            )
            val = (pd.concat([
                val[val.Y == major_class].sample(
                    n=len(val[val.Y == minor_class]),
                    replace=False,
                    random_state=seed,
                ),
                val[val.Y == minor_class],
            ]).sample(frac=1, replace=False,
                      random_state=seed).reset_index(drop=True))
        else:
            print(" Oversample of minority class is used. ",
                  flush=True,
                  file=sys.stderr)
            val = (pd.concat([
                val[val.Y == minor_class].sample(
                    n=len(val[val.Y == major_class]),
                    replace=True,
                    random_state=seed,
                ),
                val[val.Y == major_class],
            ]).sample(frac=1, replace=False,
                      random_state=seed).reset_index(drop=True))
        return val



"""
Dataloader class for resource datasets

TODO: we should create a way for task-specific loaders to leverage this in the backend via the dsl config.
TODO: for example, the tchard dataset uses this loader, but it is a multi_pred task (tct-epitope)
"""

import pandas as pd

from .. import base_dataset
from .. import metadata
from ..utils import load
from ..dataset_configs import config, config_map


class DataLoader(base_dataset.DataLoader, dict):
    """
    Dataloader class for resource datasets
    """

    def __init__(self, name, path="./data", dataset_names=None):
        dataset_names = dataset_names if dataset_names is not None else metadata.dataset_names[
            "Resource"]
        names = None
        if type(name) == list:
            names = name
        elif type(name) == str:
            assert name in metadata.resources, "{} is not a resource {}".format(
                name, metadata.resources.keys())
            names = [n for n in metadata.resources[name]["all"]]
        else:
            raise ValueError("expected a list of names or a single name. Got",
                             name)
        assert type(names) == list, "list expected"
        for n in names:
            self[n] = load.resource_dataset_load(n, path, dataset_names)
        for nsplit in metadata.resources[name].get("splits", []):
            self[nsplit] = {"splits": self[nsplit]}
        if "splits_raw" in metadata.resources[name]:
            self.df_key = "all_data"
            self[self.df_key] = pd.concat([
                load.resource_dataset_load(dfname, path, dataset_names)
                for dfname in metadata.resources[name].get("all", [])
            ])
            self.split_key = "splits"
            self[self.split_key] = metadata.resources[name].get(
                "splits_raw", [])
            # for now, we have hard-coded config embedded in metadata; should implement dsl config support for this case as well.
            self.config = metadata.resources[name].get("config")
            # now just need to load the actual data into the splits
            for split_type, labeled_splits in self[self.split_key].items():
                for label, keyed_splits in labeled_splits.items():
                    for key, split_name in keyed_splits.items():
                        if type(split_name) == str:
                            splitdf = load.resource_dataset_load(
                                split_name, path, dataset_names)
                            # apply config
                            splitdf["Y"] = splitdf[self.config["Y"]]
                            # store dataframe
                            self[self.
                                 split_key][split_type][label][key] = splitdf
                        else:
                            assert type(split_name) == pd.DataFrame, type(
                                split_name)

            # apply label
            if "Y" not in self[self.df_key].columns:  # for repeat loader runs
                self[self.df_key]["Y"] = self[self.df_key][self.config["Y"]]
            return  # no further data processing needed. TODO: can support dsl configs as well here.

        # raise Exception("keys", self.keys())
        self.config = config_map.ConfigMap().get(name)
        self.config = self.config() if self.config is not None else None
        assert self.config is not None, "resource.DataLoader requires a corresponding config"
        assert isinstance(
            self.config, config.ResourceConfig
        ), "resource.DataLoader requires a ResourceConfig, got {}".format(
            type(self.config))
        # run transformations
        self.config.loader_setup_callback(self)
        # run data transformations
        if self.config.processing_callback is not None:
            self[self.config.df_key] = self.config.processing_callback(
                self[self.config.df_key])
        else:
            self[self.config.df_key] = self[self.config.df_key]

    def get_data(self, df_key=None, **kwargs):
        # TODO: can call parent's get_data(**kwargs) function if dataset not pre-loaded
        df_key = df_key or self.config.df_key if type(
            self.config) != dict else self.df_key
        assert df_key in self, "{} key hasn't been set in the loader, please set it by using the resource.DataLoader".format(
            df_key)
        return self[df_key]

    def get_split(self, split_key=None, **kwargs):
        # TODO: can call parent's get_split(**kwargs) function if splits not pre-loaded
        split_key = split_key or self.config.split_key if type(
            self.config) != dict else self.split_key
        assert split_key in self, "{} key hasn't been set in the loader, please set it by using the resource.DataLoader".format(
            split_key)
        return self[split_key]



import pandas as pd
import numpy as np
import os, sys, json
import warnings
from packaging import version
import pkg_resources

warnings.filterwarnings("ignore")

from .utils import fuzzy_search, oracle_load, receptor_load
from .metadata import (
    download_oracle_names,
    oracle_names,
    distribution_oracles,
    docking_oracles,
    download_receptor_oracle_name,
    docking_target_info,
)

SKLEARN_VERSION = version.parse(
    pkg_resources.get_distribution("scikit-learn").version)


def _normalize_docking_score(raw_score):
    return 1 / (1 + np.exp((raw_score + 7.5)))


class Oracle:
    """the oracle class to retrieve any oracle given by query name

    Args:
        name (str): the name of the oracle
        target_smiles (None, optional): target smiles for some meta-oracles
        num_max_call (None, optional): number of maximum calls for oracle, used by docking group
        **kwargs: additional parameters for some oracles
    """

    def __init__(self, name, target_smiles=None, num_max_call=None, **kwargs):
        """Summary"""
        self.target_smiles = target_smiles
        self.kwargs = kwargs
        self.normalize = lambda x: x
        name = fuzzy_search(name, oracle_names)
        if name == "drd3_docking":
            name = "3pbl_docking"
        if name == "drd3_docking_normalize":
            name = "3pbl_docking_normalize"
        if name in download_oracle_names:
            if name in ["jnk3", "gsk3b", "drd2"]:
                if SKLEARN_VERSION >= version.parse("0.24.0"):
                    name += "_current"
            ### download
            ##### e.g., jnk, gsk, drd2, ...
            self.name = oracle_load(name)
        elif name in download_receptor_oracle_name:
            ## '1iep_docking', '2rgp_docking', '7l11_docking', 'drd3_docking', '3pbl_docking',
            pdbid = name.split("_")[0]
            self.name = receptor_load(pdbid)
            self.pdbid = self.name
            self.name += "_docking"
            if "normalize" in name:
                self.name += "_normalize"
                self.normalize = _normalize_docking_score
        else:
            self.name = name
        self.evaluator_func = None
        self.assign_evaluator()
        self.num_called = 0

        if num_max_call is not None:
            self.num_max_call = num_max_call
        else:
            self.num_max_call = None

    def assign_evaluator(self):
        """assign the specific oracle function given by query oracle name"""
        self.default_property = 0.0
        if self.name == "logp":
            from .chem_utils import penalized_logp

            self.evaluator_func = penalized_logp
        elif self.name == "qed":
            from .chem_utils import qed

            self.evaluator_func = qed
        # elif self.name == 'drd2':
        elif "drd2" in self.name:
            from .chem_utils import drd2

            self.evaluator_func = drd2
        elif self.name == "cyp3a4_veith":
            from .chem_utils import cyp3a4_veith

            self.evaluator_func = cyp3a4_veith
        elif self.name == "sa":
            from .chem_utils import SA

            self.evaluator_func = SA
        # elif self.name == 'gsk3b':
        elif "gsk3b" in self.name:
            from .chem_utils import gsk3b

            oracle_object = gsk3b
            self.evaluator_func = oracle_object
        # elif self.name == 'jnk3':
        elif "jnk3" in self.name:
            from .chem_utils import jnk3

            oracle_object = jnk3()
            self.evaluator_func = oracle_object
        elif self.name == "similarity_meta":
            from .chem_utils import similarity_meta

            self.evaluator_func = similarity_meta(
                target_smiles=self.target_smiles, **self.kwargs)
        elif self.name == "rediscovery_meta":
            from .chem_utils import rediscovery_meta

            self.evaluator_func = rediscovery_meta(
                target_smiles=self.target_smiles, **self.kwargs)
        elif self.name == "isomer_meta":
            from .chem_utils import isomer_meta

            self.evaluator_func = isomer_meta(target_smiles=self.target_smiles,
                                              **self.kwargs)
        elif self.name == "median_meta":
            from .chem_utils import median_meta

            self.evaluator_func = median_meta(
                target_smiles_1=self.target_smiles[0],
                target_smiles_2=self.target_smiles[1],
                **self.kwargs)
        elif self.name == "rediscovery":
            from .chem_utils import (
                celecoxib_rediscovery,
                troglitazone_rediscovery,
                thiothixene_rediscovery,
            )

            self.evaluator_func = {
                "Celecoxib": celecoxib_rediscovery,
                "Troglitazone": troglitazone_rediscovery,
                "Thiothixene": thiothixene_rediscovery,
            }
        elif self.name == "celecoxib_rediscovery":
            from .chem_utils import celecoxib_rediscovery

            self.evaluator_func = celecoxib_rediscovery
        elif self.name == "troglitazone_rediscovery":
            from .chem_utils import troglitazone_rediscovery

            self.evaluator_func = troglitazone_rediscovery
        elif self.name == "thiothixene_rediscovery":
            from .chem_utils import thiothixene_rediscovery

            self.evaluator_func = thiothixene_rediscovery
        elif self.name == "similarity":
            from .chem_utils import (
                aripiprazole_similarity,
                albuterol_similarity,
                mestranol_similarity,
            )

            self.evaluator_func = {
                "Aripiprazole": aripiprazole_similarity,
                "Albuterol": albuterol_similarity,
                "Mestranol": mestranol_similarity,
            }
        elif self.name == "aripiprazole_similarity":
            from .chem_utils import aripiprazole_similarity

            self.evaluator_func = aripiprazole_similarity
        elif self.name == "albuterol_similarity":
            from .chem_utils import albuterol_similarity

            self.evaluator_func = albuterol_similarity
        elif self.name == "mestranol_similarity":
            from .chem_utils import mestranol_similarity

            self.evaluator_func = mestranol_similarity
        elif self.name == "median":
            from .chem_utils import median1, median2

            self.evaluator_func = {"Median 1": median1, "Median 2": median2}
        elif self.name == "median1":
            from .chem_utils import median1

            self.evaluator_func = median1
        elif self.name == "median2":
            from .chem_utils import median2

            self.evaluator_func = median2
        elif self.name == "mpo":
            from .chem_utils import (
                osimertinib_mpo,
                fexofenadine_mpo,
                ranolazine_mpo,
                perindopril_mpo,
                amlodipine_mpo,
                sitagliptin_mpo,
                zaleplon_mpo,
            )

            self.evaluator_func = {
                "Osimertinib": osimertinib_mpo,
                "Fexofenadine": fexofenadine_mpo,
                "Ranolazine": ranolazine_mpo,
                "Perindopril": perindopril_mpo,
                "Amlodipine": amlodipine_mpo,
                "Sitagliptin": sitagliptin_mpo,
                "Zaleplon": zaleplon_mpo,
            }
        elif self.name == "osimertinib_mpo":
            from .chem_utils import osimertinib_mpo

            self.evaluator_func = osimertinib_mpo
        elif self.name == "fexofenadine_mpo":
            from .chem_utils import fexofenadine_mpo

            self.evaluator_func = fexofenadine_mpo
        elif self.name == "ranolazine_mpo":
            from .chem_utils import ranolazine_mpo

            self.evaluator_func = ranolazine_mpo
        elif self.name == "perindopril_mpo":
            from .chem_utils import perindopril_mpo

            self.evaluator_func = perindopril_mpo
        elif self.name == "amlodipine_mpo":
            from .chem_utils import amlodipine_mpo

            self.evaluator_func = amlodipine_mpo
        elif self.name == "sitagliptin_mpo_prev":
            from .chem_utils import sitagliptin_mpo_prev

            self.evaluator_func = sitagliptin_mpo_prev
        elif self.name == "sitagliptin_mpo":
            from .chem_utils import sitagliptin_mpo

            self.evaluator_func = sitagliptin_mpo
        elif self.name == "zaleplon_mpo_prev":
            from .chem_utils import zaleplon_mpo_prev

            self.evaluator_func = zaleplon_mpo_prev
        elif self.name == "zaleplon_mpo":
            from .chem_utils import zaleplon_mpo

            self.evaluator_func = zaleplon_mpo
        elif self.name == "valsartan_smarts":
            from .chem_utils import valsartan_smarts

            self.evaluator_func = valsartan_smarts
        elif self.name == "hop":
            from .chem_utils import deco_hop, scaffold_hop

            self.evaluator_func = {
                "Deco Hop": deco_hop,
                "Scaffold Hop": scaffold_hop
            }
        elif self.name == "deco_hop":
            from .chem_utils import deco_hop

            self.evaluator_func = deco_hop
        elif self.name == "scaffold_hop":
            from .chem_utils import scaffold_hop

            self.evaluator_func = scaffold_hop
        elif self.name == "isomers_c7h8n2o2":
            from .chem_utils import isomers_c7h8n2o2

            self.evaluator_func = isomers_c7h8n2o2
        elif self.name == "isomers_c9h10n2o2pf2cl":
            from .chem_utils import isomers_c9h10n2o2pf2cl

            self.evaluator_func = isomers_c9h10n2o2pf2cl
        elif self.name == "isomers_c11h24":
            from .chem_utils import isomers_c11h24

            self.evaluator_func = isomers_c11h24
        elif self.name == "isomers":
            from .chem_utils import isomers_c7h8n2o2, isomers_c9h10n2o2pf2cl

            self.evaluator_func = {
                "c7h8n2o2": isomers_c7h8n2o2,
                "c9h10n2o2pf2cl": isomers_c9h10n2o2pf2cl,
            }
        elif self.name == "askcos":  #### synthetic analysis
            from .chem_utils import askcos

            self.evaluator_func = askcos
        elif self.name == "ibm_rxn":
            from .chem_utils import ibm_rxn

            self.evaluator_func = ibm_rxn
        elif self.name == "molecule_one_synthesis":
            from .chem_utils import molecule_one_retro

            self.evaluator_func = molecule_one_retro(**self.kwargs)
        elif self.name == "pyscreener":
            from .chem_utils import PyScreener_meta

            self.evaluator_func = PyScreener_meta(**self.kwargs)
        elif self.name == "docking_score":
            from .chem_utils import Vina_smiles

            self.evaluator_func = Vina_smiles(**self.kwargs)
        elif self.name == "drd3_docking_vina" or self.name == "3pbl_docking_vina":

            from .chem_utils import Vina_smiles

            pdbid = "3pbl"
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = Vina_smiles(
                receptor_pdbqt_file="./oracle/" + pdbid + ".pdbqt",
                center=center,
                box_size=boxsize,
            )

        elif (self.name == "drd3_docking" or self.name == "3pbl_docking" or
              self.name == "drd3_docking_normalize" or
              self.name == "3pbl_docking_normalize"):

            from .chem_utils import PyScreener_meta

            pdbid = "3pbl"
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = PyScreener_meta(
                receptor_pdb_file="./oracle/" + pdbid + ".pdb",
                box_center=center,
                box_size=boxsize,
            )

        elif self.name == "1iep_docking_vina":
            from .chem_utils import Vina_smiles

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = Vina_smiles(
                receptor_pdbqt_file="./oracle/" + pdbid + ".pdbqt",
                center=center,
                box_size=boxsize,
            )
        elif self.name == "1iep_docking" or self.name == "1iep_docking_normalize":
            from .chem_utils import PyScreener_meta

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = PyScreener_meta(
                receptor_pdb_file="./oracle/" + pdbid + ".pdb",
                box_center=center,
                box_size=boxsize,
            )
        elif self.name == "2rgp_docking_vina":
            from .chem_utils import Vina_smiles

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = Vina_smiles(
                receptor_pdbqt_file="./oracle/" + pdbid + ".pdbqt",
                center=center,
                box_size=boxsize,
            )
        elif self.name == "2rgp_docking" or self.name == "2rgp_docking_normalize":
            from .chem_utils import PyScreener_meta

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = PyScreener_meta(
                receptor_pdb_file="./oracle/" + pdbid + ".pdb",
                box_center=center,
                box_size=boxsize,
            )
        elif self.name == "3eml_docking_vina":
            from .chem_utils import Vina_smiles

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = Vina_smiles(
                receptor_pdbqt_file="./oracle/" + pdbid + ".pdbqt",
                center=center,
                box_size=boxsize,
            )
        elif self.name == "3eml_docking" or self.name == "3eml_docking_normalize":
            from .chem_utils import PyScreener_meta

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = PyScreener_meta(
                receptor_pdb_file="./oracle/" + pdbid + ".pdb",
                box_center=center,
                box_size=boxsize,
            )
        elif self.name == "3ny8_docking_vina":
            from .chem_utils import Vina_smiles

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = Vina_smiles(
                receptor_pdbqt_file="./oracle/" + pdbid + ".pdbqt",
                center=center,
                box_size=boxsize,
            )
        elif self.name == "3ny8_docking" or self.name == "3ny8_docking_normalize":
            from .chem_utils import PyScreener_meta

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = PyScreener_meta(
                receptor_pdb_file="./oracle/" + pdbid + ".pdb",
                box_center=center,
                box_size=boxsize,
            )
        elif self.name == "4rlu_docking_vina":
            from .chem_utils import Vina_smiles

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = Vina_smiles(
                receptor_pdbqt_file="./oracle/" + pdbid + ".pdbqt",
                center=center,
                box_size=boxsize,
            )
        elif self.name == "4rlu_docking" or self.name == "4rlu_docking_normalize":
            from .chem_utils import PyScreener_meta

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = PyScreener_meta(
                receptor_pdb_file="./oracle/" + pdbid + ".pdb",
                box_center=center,
                box_size=boxsize,
            )
        elif self.name == "4unn_docking_vina":
            from .chem_utils import Vina_smiles

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = Vina_smiles(
                receptor_pdbqt_file="./oracle/" + pdbid + ".pdbqt",
                center=center,
                box_size=boxsize,
            )
        elif self.name == "4unn_docking" or self.name == "4unn_docking_normalize":
            from .chem_utils import PyScreener_meta

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = PyScreener_meta(
                receptor_pdb_file="./oracle/" + pdbid + ".pdb",
                box_center=center,
                box_size=boxsize,
            )
        elif self.name == "5mo4_docking_vina":
            from .chem_utils import Vina_smiles

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = Vina_smiles(
                receptor_pdbqt_file="./oracle/" + pdbid + ".pdbqt",
                center=center,
                box_size=boxsize,
            )
        elif self.name == "5mo4_docking" or self.name == "5mo4_docking_normalize":
            from .chem_utils import PyScreener_meta

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = PyScreener_meta(
                receptor_pdb_file="./oracle/" + pdbid + ".pdb",
                box_center=center,
                box_size=boxsize,
            )
        elif self.name == "7l11_docking_vina":
            from .chem_utils import Vina_smiles

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = Vina_smiles(
                receptor_pdbqt_file="./oracle/" + pdbid + ".pdbqt",
                center=center,
                box_size=boxsize,
            )
        elif self.name == "7l11_docking" or self.name == "7l11_docking_normalize":
            from .chem_utils import PyScreener_meta

            pdbid = self.name.split("_")[0]
            center = docking_target_info[pdbid]["center"]
            boxsize = docking_target_info[pdbid]["size"]
            self.evaluator_func = PyScreener_meta(
                receptor_pdb_file="./oracle/" + pdbid + ".pdb",
                box_center=center,
                box_size=boxsize,
            )
        # elif self.name == '3pbl_docking':
        # 	from .chem_utils import Vina_smiles
        # 	pdbid = self.name.split('_')[0]
        # 	center = docking_target_info[pdbid]['center']
        # 	boxsize = docking_target_info[pdbid]['size']
        # 	self.evaluator_func = Vina_smiles(receptor_pdbqt_file='./oracle/'+pdbid+'.pdbqt',
        # 									  center = center,
        # 									  box_size = boxsize)
        elif self.name == "uniqueness":
            from .chem_utils import uniqueness

            self.evaluator_func = uniqueness
        elif self.name == "validity":
            from .chem_utils import validity

            self.evaluator_func = validity
        elif self.name == "diversity":
            from .chem_utils import diversity

            self.evaluator_func = diversity
        elif self.name == "novelty":
            from .chem_utils import novelty

            self.evaluator_func = novelty
        elif self.name == "fcd_distance":
            from .chem_utils import fcd_distance

            self.evaluator_func = fcd_distance
        elif self.name == "kl_divergence":
            from .chem_utils import kl_divergence

            self.evaluator_func = kl_divergence

        elif self.name == "smina":
            from .chem_utils import smina

            self.evaluator_func = smina

        else:
            return

    def __call__(self, *args, **kwargs):
        """call the oracle function on SMILES to genenerate scores

        Args:
            *args: a list of SMILES/a string of SMILES
            **kwargs: additional parameters for some oracles

        Returns:
            float/list: the oracle score(s) for a single/list of SMILES

        Raises:
            ValueError: reached number of maximum calls if set and has queries the oracle more than the internal call counters
        """
        if self.name in distribution_oracles:
            ## 'novelty', 'diversity', 'uniqueness', 'validity', 'fcd_distance', 'kl_divergence'
            return self.evaluator_func(*args, **kwargs)

        if self.name in docking_oracles:
            return self.evaluator_func(*args, **kwargs)

        from rdkit import Chem

        smiles_lst = args[0]
        if self.name == "molecule_one_synthesis":
            return self.evaluator_func(*args, **kwargs)

        if type(smiles_lst) == list:
            nonvalid_smiles_idx_lst, valid_smiles_lst, valid_smiles_idx_lst = [], [], []
            NN = len(smiles_lst)
            for idx, smiles in enumerate(smiles_lst):
                if Chem.MolFromSmiles(smiles) == None:
                    nonvalid_smiles_idx_lst.append(idx)
                else:
                    valid_smiles_idx_lst.append(idx)
                    valid_smiles_lst.append(smiles)
            smiles_lst = valid_smiles_lst

            self.num_called += len(smiles_lst)
            if self.num_max_call is not None:
                if self.num_max_call < self.num_called:
                    self.num_called -= len(smiles_lst)
                    raise ValueError(
                        "The maximum number of evaluator call is reached! The maximum is: "
                        + str(self.num_max_call) +
                        ". The current requested call (plus accumulated calls) is: "
                        + str(self.num_called + len(smiles_lst)))

            #### evaluator for single molecule,
            #### the input of __call__ is a single smiles OR list of smiles
            if isinstance(self.evaluator_func, dict):
                all_ = {}
                for i, fct in self.evaluator_func.items():
                    results_lst = []
                    for smiles in smiles_lst:
                        results_lst.append(fct(smiles, *(args[1:]), **kwargs))
                    all_[i] = results_lst
                return all_
            else:
                results_lst = []

                if not self.name == "docking_score":
                    for smiles in smiles_lst:
                        results_lst.append(
                            self.normalize(
                                self.evaluator_func(smiles, *(args[1:]),
                                                    **kwargs)))
                else:
                    results_lst = []
                    for smiles in smiles_lst:
                        try:
                            results = self.evaluator_func([smiles], *(args[1:]),
                                                          **kwargs)
                            results = results[0]
                        except:
                            results = self.default_property
                        results_lst.append(results)
                    # results_lst = self.evaluator_func(smiles_lst, *(args[1:]), **kwargs)
                    results_lst = [self.normalize(i) for i in results_lst]
                all_results_lst = [self.default_property for i in range(NN)]
                for idx, result in zip(valid_smiles_idx_lst, results_lst):
                    all_results_lst[idx] = result
                return all_results_lst

        else:  ### a string of SMILES
            if Chem.MolFromSmiles(smiles_lst) == None:
                return self.default_property

            self.num_called += 1
            if self.num_max_call is not None:
                if self.num_max_call < self.num_called:
                    self.num_called -= 1
                    raise ValueError(
                        "The maximum number of evaluator call is reached! The maximum is: "
                        + str(self.num_max_call) +
                        ". The current requested call (plus accumulated calls) is: "
                        + str(self.num_called + 1))

            ## a single smiles
            if type(self.evaluator_func) == dict:
                all_ = {}
                for i, fct in self.evaluator_func.items():
                    all_[i] = fct(*args, **kwargs)
                return all_
            else:
                try:
                    score = self.evaluator_func(*args, **kwargs)
                except:
                    score = self.default_property
                return self.normalize(score)
                # return self.normalize(self.evaluator_func(*args, **kwargs))



# -*- coding: utf-8 -*-
# Author: TDC Team
# License: MIT

from .base_group import BenchmarkGroup


class admet_group(BenchmarkGroup):
    """Create ADMET Group Class object.

    Args:
            path (str, optional): the path to store/retrieve the ADMET group datasets.
    """

    def __init__(self, path="./data"):
        """Create an ADMET benchmark group class."""
        super().__init__(name="ADMET_Group", path=path, file_format="csv")



"""
Class encapsulating protein data processing functions. Also supports running them in sequence.
Goal is to make it easier to integrate custom datasets not yet in TDC format.
"""

from Bio import Entrez, SeqIO
import mygene
from pandas import DataFrame
import requests

from .data_feature_generator import DataFeatureGenerator


class ProteinFeatureGenerator(DataFeatureGenerator):
    """
    Class encapsulating protein data processing functions. Also supports running them in sequence.
    Goals are to make it easier to integrate custom datasets not yet in TDC format.
    Note: for running in sequence, this class inherits from data_processing_utils.DataFeatureGenerator
    """

    _SEQUENCE_MAP = {
        "ACE2": (
            "protein-coding",
            "MSSSSWLLLSLVAVTAAQSTIEEQAKTFLDKFNHEAEDLFYQSSLASWNYNTNITEENVQNMNNAGDKWSAFLKEQSTLAQMYPLQEIQNLTVKLQLQALQQNGSSVLSEDKSKRLNTILNTMSTIYSTGKVCNPDNPQECLLLEPGLNEIMANSLDYNERLWAWESWRSEVGKQLRPLYEEYVVLKNEMARANHYEDYGDYWRGDYEVNGVDGYDYSRGQLIEDVEHTFEEIKPLYEHLHAYVRAKLMNAYPSYISPIGCLPAHLLGDMWGRFWTNLYSLTVPFGQKPNIDVTDAMVDQAWDAQRIFKEAEKFFVSVGLPNMTQGFWENSMLTDPGNVQKAVCHPTAWDLGKGDFRILMCTKVTMDDFLTAHHEMGHIQYDMAYAAQPFLLRNGANEGFHEAVGEIMSLSAATPKHLKSIGLLSPDFQEDNETEINFLLKQALTIVGTLPFTYMLEKWRWMVFKGEIPKDQWMKKWWEMKREIVGVVEPVPHDETYCDPASLFHVSNDYSFIRYYTRTLYQFQFQEALCQAAKHEGPLHKCDISNSTEAGQKLFNMLRLGKSEPWTLALENVVGAKNMNVRPLLNYFEPLFTWLKDQNKNSFVGWSTDWSPYADQSIKVRISLKSALGDKAYEWNDNEMYLFRSSVAYAMRQYFLKVKNQMILFGEEDVRVANLKPRISFNFFVTAPKNVSDIIPRTEVEKAIRMSRSRINDAFRLNDNSLEFLGIQPTLGPPNQPPVSIWLIVFGVVMGVIVVGIVILIFTGIRDRKKKNKARSGENPYASIDISKGENNPGFQNTDDVQTSF"
        ),
        "12CA5": ("protein-coding", "YPYDVPDYA")
    }

    @classmethod
    def get_ncrna_sequence(cls, ncrna_id):
        """
        Fetches the nucleotide sequence for a given non-coding RNA ID from NCBI.

        Args:
        ncrna_id (str): The NCBI identifier for the non-coding RNA.

        Returns:
        str: The nucleotide sequence of the non-coding RNA, or a message if not found.
        """
        # Provide your email to NCBI to let them know who you are
        Entrez.email = "alejandro_velez-arce@hms.harvard.edu"

        try:
            # Fetch the sequence using efetch
            handle = Entrez.efetch(db="nucleotide",
                                   id=ncrna_id,
                                   rettype="fasta",
                                   retmode="text")
            record = SeqIO.read(handle, "fasta")
            handle.close()
            return str(record.seq)
        except Exception as e:
            return "Failed to retrieve sequence: " + str(e)

    @classmethod
    def get_amino_acid_sequence(cls, uniprot_id):
        """
        Fetches the amino acid sequence for a given UniProt ID.

        Args:
        uniprot_id (str): The UniProt ID for the protein.

        Returns:
        str: The amino acid sequence of the protein.
        """
        url = f"https://www.uniprot.org/uniprot/{uniprot_id}.fasta"
        response = requests.get(url)

        if response.status_code == 200:
            fasta_content = response.text
            # Split the FASTA content into lines and discard the first line (header)
            sequence_lines = fasta_content.split('\n')[1:]
            # Join the lines to get the sequence
            sequence = ''.join(sequence_lines)
            return sequence
        else:
            return "Failed to retrieve sequence. Status code: " + str(
                response.status_code)

    @classmethod
    def get_protein_sequence(cls, gene_name: str) -> str:
        """
        Retrieves protein sequence for the given gene name.
        
        Args:
            gene_name (str): Gene name.
            
        Returns:
            str: Protein amino acid sequence.
        """
        assert isinstance(gene_name, str), (type(gene_name), gene_name)
        mg = mygene.MyGeneInfo()
        # Query MyGene.info for the given gene name
        # You might need to adjust the fields based on the gene's specifics
        # 'fields': 'proteins' might vary depending on the data available for your gene
        if gene_name in cls._SEQUENCE_MAP:  # genes for which mygene seems to be mislabled
            return cls._SEQUENCE_MAP[gene_name][1]
        gene_info = mg.query(gene_name, fields='all', species='human')
        try:
            # Attempt to extract the protein sequence
            # The path to the protein sequence might need adjustment based on the response structure
            if gene_info['hits'][0]["type_of_gene"].lower() == "ncrna":
                ncbi_id = gene_info['hits'][0]['entrezgene']
                return cls.get_ncrna_sequence(ncbi_id)

            protid = gene_info['hits'][0]["uniprot"]["Swiss-Prot"]
            return cls.get_amino_acid_sequence(protid)
        except (IndexError, KeyError):
            # Handle cases where the protein sequence is not found
            return "Protein sequence not found for the given gene name."

    @classmethod
    def get_type_of_gene(cls, gene_name: str) -> str:
        assert isinstance(gene_name, str), (type(gene_name), gene_name)
        mg = mygene.MyGeneInfo()
        # Query MyGene.info for the given gene name
        # You might need to adjust the fields based on the gene's specifics
        # 'fields': 'proteins' might vary depending on the data available for your gene
        if gene_name in cls._SEQUENCE_MAP:
            return cls._SEQUENCE_MAP[gene_name][0]
        gene_info = mg.query(gene_name, fields='all', species='human')
        try:
            return gene_info["hits"][0][
                "type_of_gene"] if gene_name != "12CA5" else "protein-coding"
        except:
            return "Could not find type of gene"

    @classmethod
    def insert_protein_sequence(cls, dataset: DataFrame,
                                gene_column: str) -> DataFrame:
        """
        Inserts protein sequence for each gene in the given DataFrame.
        
        Args:
            gene_df (pd.DataFrame): Input DataFrame containing gene identifiers.
            gene_column (str): Column name in gene_df that contains gene identifiers.
            
        Returns:
            pd.DataFrame: DataFrame with an additional column for protein sequences.
        """

        def helper(gene_name):
            if gene_name in memo:
                return memo[gene_name]
            memo[gene_name] = cls.get_protein_sequence(gene_name)
            return memo[gene_name]

        def helper_type(gene_name):
            if gene_name in memo_type:
                return memo_type[gene_name]
            memo_type[gene_name] = cls.get_type_of_gene(gene_name)
            return memo_type[gene_name]

        memo: dict = {}  # To store already computed values
        memo_type: dict = {}
        # Check if gene_column exists in gene_df
        if gene_column not in dataset.columns:
            raise ValueError(f"{gene_column} does not exist in the DataFrame.")

        # Ensure the DataFrame index is aligned
        # gene_df = gene_df.reset_index(drop=True)

        # Retrieve protein sequences for each gene and store them in a new column
        new_col = dataset[gene_column].apply(helper).tolist()
        assert len(new_col) == len(dataset[gene_column]), (new_col,
                                                           dataset[gene_column])
        dataset['protein_or_rna_sequence'] = new_col
        dataset["gene_type"] = dataset[gene_column].apply(helper_type).tolist()

        return dataset



from .config import DatasetConfig
from ..feature_generators.protein_feature_generator import ProteinFeatureGenerator


class BrownProteinPeptideConfig(DatasetConfig):
    """Configuration for the brown-protein-peptide datasets"""

    def __init__(self):
        super(BrownProteinPeptideConfig, self).__init__(
            dataset_name="brown_mdm2_ace2_12ca5",
            data_processing_class=ProteinFeatureGenerator,
            functions_to_run=[
                "autofill_identifier", "create_range", "insert_protein_sequence"
            ],
            args_for_functions=[{
                "autofill_column": "Name",
                "key_column": "Sequence",
            }, {
                "column": "KD (nM)",
                "keys": ["Putative binder"],
                "subs": [0]
            }, {
                "gene_column": "Protein Target"
            }],
            var_map={
                "X1": "Sequence",
                "X2": "protein_or_rna_sequence",
                "ID1": "Name",
                "ID2": "Protein Target",
            },
        )



from ..utils import general_load
from ..utils.load import download_wrapper, load_json_from_txt_file, zip_data_download_wrapper

import pandas as pd
import os
import torch


class PINNACLE:
    """
    PINNACLE is a class for loading and manipulating the PINNACLE networks and embeddings.
    """

    def __init__(self, path="./data"):
        self.ppi_name = "pinnacle_global_ppi_edgelist"
        self.cell_tissue_mg_name = "cell_tissue_mg_edgelist"
        self.ppi = general_load(self.ppi_name, path, " ")
        self.ppi.columns = ["Protein A", "Protein B"]
        self.cell_tissue_mg = general_load(
            self.cell_tissue_mg_name, path,
            "\t")  # use tab as names were left with spaces
        self.cell_tissue_mg.columns = ["Tissue", "Cell"]
        self.embeds_name = "pinnacle_protein_embed"
        self.embeds_name = download_wrapper(self.embeds_name, path,
                                            self.embeds_name)
        self.embeds = torch.load(os.path.join(path, self.embeds_name + ".pth"))
        self.keys = load_json_from_txt_file("pinnacle_labels_dict", path)

    def get_ppi(self):
        return self.ppi

    def get_mg(self):
        return self.cell_tissue_mg

    def get_embeds_raw(self):
        return self.embeds

    def get_keys(self):
        protein_names_celltypes = [
            p for p in zip(self.keys["Cell Type"], self.keys["Name"])
            if not (p[0].startswith("BTO") or p[0].startswith("CCI") or
                    p[0].startswith("Sanity"))
        ]
        proteins = pd.DataFrame.from_dict({
            "target": [n for _, n in protein_names_celltypes],
            "cell type": [c for c, _ in protein_names_celltypes]
        })
        proteins.drop_duplicates()
        return proteins

    def get_embeds(self):
        prots = self.get_keys()
        emb = self.get_embeds_raw()
        x = {}
        ctr = 0
        for _, v in emb.items():
            if isinstance(v, torch.Tensor):
                if v.size()[0] == 1:
                    k = "--".join(prots.iloc[ctr])
                    ctr += 1
                    x[k] = v.detach().numpy()
                    continue
                for t in v:
                    assert len(t.size()) == 1, t.size()
                    k = "--".join(prots.iloc[ctr])
                    ctr += 1
                    x[k] = t.detach().numpy()
            else:
                raise Exception("encountered non-tensor")
        assert len(x) == len(prots), "dict len {} vs keys length {}".format(
            len(x), len(prots))
        df = pd.DataFrame.from_dict(x)
        df = df.transpose()
        assert len(df) == len(
            x), "dims not mantained when translated to pandas. {} vs {}".format(
                len(df), len(x))
        return df

    def get_exp_data(self, seed=1, split="train"):
        if split not in ["train", "val", "test"]:
            raise ValueError("{} not a valid split".format(split))
        filename = "pinnacle_output{}".format(seed)
        # clean data directory
        file_list = os.listdir("./data")
        for file in file_list:
            try:
                os.remove(os.path.join("./data", file))
            except:
                continue
        print("downloading pinancle zip data...")
        zip_data_download_wrapper(
            filename, "./data",
            ["pinnacle_output{}".format(x) for x in range(1, 11)])
        print("success!")
        # Get non-csv files and remove them
        non_csv_files = [
            f for f in os.listdir("./data") if not f.endswith(".csv")
        ]
        for x in non_csv_files:
            try:
                os.remove("./data/{}".format(x))
            except:
                continue
        # Get a list of all CSV files in the unzipped folder
        csv_files = [f for f in os.listdir("./data") if f.endswith(".csv")]
        if not csv_files:
            raise Exception("no csv")
        x = []
        print("iterating over csv files...")
        for file in csv_files:
            print("got file {}".format(file))
            if "_{}_".format(split) not in file:
                os.remove("./data/{}".format(file))
                continue
            print("reading into pandas...")
            df = pd.read_csv("./data/{}".format(file))
            cell = file.split("_")[-1]
            cell = cell.split(".")[0]
            df["cell_type_label"] = cell
            disease = "IBD" if "3767" in file else "RA"
            df["disease"] = disease
            x.append(df)
            os.remove("./data/{}".format(file))
        return pd.concat(x, axis=0, ignore_index=True)


import pandas as pd
import numpy as np
import os, sys, json
import warnings

warnings.filterwarnings("ignore")

from .utils import bm_group_load, print_sys, fuzzy_search
from .utils import (
    create_fold,
    create_fold_setting_cold,
    create_combination_split,
    create_fold_time,
    create_scaffold_split,
    create_group_split,
)
from .metadata import (
    get_task2category,
    bm_metric_names,
    benchmark_names,
    bm_split_names,
    docking_target_info,
)
from .evaluator import Evaluator


class BenchmarkGroup:

    def __init__(
        self,
        name,
        path="./data",
        file_format="csv",
        pyscreener_path=None,
        num_workers=None,
        num_cpus=None,
        num_max_call=5000,
    ):
        """
        -- PATH
                -- ADMET_Benchmark
                        -- HIA_Hou
                                -- train_val.csv
                                -- test.csv
                        -- Caco2_Wang
                                -- train_val.csv
                                -- test.csv
                        ....
        from tdc import BenchmarkGroup
        group = BenchmarkGroup(name = 'ADMET_Group', path = 'data/')
        predictions = {}

        for benchmark in group:
           name = benchmark['name']
           train_val, test = benchmark['train_val'], benchmark['test']

           # to obtain any number of train:valid split
           train, valid = group.get_train_valid_split(benchmark = name, split_type = 'default', seed = 42, frac = [0.875, 0.125])

           ## --- train your model --- ##
           predictions[name] = y_pred

        group.evaluate(predictions)
        # {'caco2_wang': 0.234, 'hia_hou': 0.786}

        benchmark = group.get('Caco2_Wang')
        train, valid, test = benchmark['train'], benchmark['valid'], benchmark['test']
        ## --- train your model --- ##
        group.evaluate(y_pred, benchmark = 'Caco2_Wang')
        # 0.234

        from tdc import BenchmarkGroup
        group = BenchmarkGroup(name = 'ADMET_Group', path = 'data/')
        predictions_list = []

        for seed in [1, 2, 3, 4, 5]:
            predictions = {}
            for benchmark in group:
                name = benchmark['name']
                train_val, test = benchmark['train_val'], benchmark['test']
                train, valid = group.get_train_valid_split(benchmark = name, split_type = 'default', seed = seed)
                ## --- train your model --- ##
                y_pred = [1] * len(test)
                predictions[name] = y_pred
            predictions_list.append(predictions)

        group.evaluate_many(predictions_list)

        file_format: csv: for admet; pkl: for drugcomb; oracle: for docking score

        """

        print_sys(
            "tdc.BenchmarkGroup will be deprecated soon. Please use tdc.benchmark_group.XXX_group and check out the examples on website!"
        )

        self.name = bm_group_load(name, path)
        self.path = os.path.join(path, self.name)
        self.datasets = benchmark_names[self.name]
        self.dataset_names = []
        self.file_format = file_format

        for task, datasets in self.datasets.items():
            for dataset in datasets:
                self.dataset_names.append(dataset)

        if self.name == "docking_group":
            if pyscreener_path is not None:
                self.pyscreener_path = pyscreener_path
            else:
                raise ValueError("Please specify pyscreener_path!")

            if (num_workers is None) and (num_cpus is None):
                ## automatic selections
                cpu_total = os.cpu_count()
                if cpu_total > 1:
                    num_cpus = 2
                else:
                    num_cpus = 1
                num_workers = int(cpu_total / num_cpus)

            self.num_workers = num_workers
            self.num_cpus = num_cpus
            self.num_max_call = num_max_call
            from .oracles import Oracle

    def __iter__(self):
        self.index = 0
        self.num_datasets = len(self.dataset_names)
        return self

    def __next__(self):
        if self.index < self.num_datasets:
            dataset = self.dataset_names[self.index]
            print_sys("--- " + dataset + " ---")

            data_path = os.path.join(self.path, dataset)
            if not os.path.exists(data_path):
                os.mkdir(data_path)
            if self.file_format == "csv":
                train = pd.read_csv(os.path.join(data_path, "train_val.csv"))
                test = pd.read_csv(os.path.join(data_path, "test.csv"))
            elif self.file_format == "pkl":
                train = pd.read_pickle(os.path.join(data_path, "train_val.pkl"))
                test = pd.read_pickle(os.path.join(data_path, "test.pkl"))
            elif self.file_format == "oracle":
                target_pdb_file = os.path.join(self.path, dataset + ".pdb")
            self.index += 1

            if self.name == "docking_group":
                from .oracles import Oracle

                oracle = Oracle(
                    name="Docking_Score",
                    software="vina",
                    pyscreener_path=self.pyscreener_path,
                    receptors=[target_pdb_file],
                    center=docking_target_info[dataset]["center"],
                    size=docking_target_info[dataset]["size"],
                    buffer=10,
                    path=data_path,
                    num_worker=self.num_workers,
                    ncpu=self.num_cpus,
                    num_max_call=self.num_max_call,
                )
                data = pd.read_csv(os.path.join(self.path, "zinc.tab"),
                                   sep="\t")
                return {"oracle": oracle, "data": data, "name": dataset}
            else:
                return {"train_val": train, "test": test, "name": dataset}
        else:
            raise StopIteration

    def get_train_valid_split(self, seed, benchmark, split_type="default"):
        if self.name == "docking_group":
            raise ValueError(
                "Docking molecule generation does not have the concept of training/testing split! Checkout the usage in tdcommons.ai !"
            )

        print_sys("generating training, validation splits...")
        dataset = fuzzy_search(benchmark, self.dataset_names)
        data_path = os.path.join(self.path, dataset)
        if self.file_format == "csv":
            train_val = pd.read_csv(os.path.join(data_path, "train_val.csv"))
        elif self.file_format == "pkl":
            train_val = pd.read_pickle(os.path.join(data_path, "train_val.pkl"))

        if split_type == "default":
            split_method = bm_split_names[self.name][dataset]
        else:
            split_method = split_type

        frac = [0.875, 0.125, 0.0]
        """
		if len(frac) == 3:
			# train:val:test split
			train_frac = frac[0]/(frac[0] + frac[1])
			valid_frac = 1 - train_frac
			frac = [train_frac, valid_frac, 0.0]
		else:
			# train:val split
			frac = [frac[0], frac[1], 0.0]
		"""
        if split_method == "scaffold":
            out = create_scaffold_split(train_val,
                                        seed,
                                        frac=frac,
                                        entity="Drug")
        elif split_method == "random":
            out = create_fold(train_val, seed, frac=frac)
        elif split_method == "combination":
            out = create_combination_split(train_val, seed, frac=frac)
        elif split_method == "group":
            out = create_group_split(train_val,
                                     seed,
                                     holdout_frac=0.2,
                                     group_column="Year")
        else:
            raise NotImplementedError
        return out["train"], out["valid"]

    def get(self, benchmark, num_max_call=5000):
        dataset = fuzzy_search(benchmark, self.dataset_names)
        data_path = os.path.join(self.path, dataset)
        if self.file_format == "csv":
            train = pd.read_csv(os.path.join(data_path, "train_val.csv"))
            test = pd.read_csv(os.path.join(data_path, "test.csv"))
        elif self.file_format == "pkl":
            train = pd.read_pickle(os.path.join(data_path, "train_val.pkl"))
            test = pd.read_pickle(os.path.join(data_path, "test.pkl"))
        elif self.file_format == "oracle":
            target_pdb_file = os.path.join(self.path, dataset + ".pdb")

        if self.name == "docking_group":
            from .oracles import Oracle

            oracle = Oracle(
                name="Docking_Score",
                software="vina",
                pyscreener_path=self.pyscreener_path,
                receptors=[target_pdb_file],
                center=docking_target_info[dataset]["center"],
                size=docking_target_info[dataset]["size"],
                buffer=10,
                path=data_path,
                num_worker=self.num_workers,
                ncpu=self.num_cpus,
                num_max_call=num_max_call,
            )
            data = pd.read_csv(os.path.join(self.path, "zinc.tab"), sep="\t")
            return {"oracle": oracle, "data": data, "name": dataset}
        else:
            return {"train_val": train, "test": test, "name": dataset}

    def evaluate(self,
                 pred,
                 true=None,
                 benchmark=None,
                 m1_api=None,
                 save_dict=True):

        if self.name == "docking_group":
            results_all = {}

            for data_name, pred_all in pred.items():
                results_max_call = {}
                for num_max_call, pred_ in pred_all.items():

                    results = {}

                    recalc = False

                    if isinstance(pred_, dict):
                        print_sys(
                            "The input is a dictionary, expected to have SMILES string as key and docking score as value!"
                        )
                        docking_scores = pred_
                        pred_ = list(pred_.keys())
                    elif isinstance(pred_, list):
                        recalc = True
                        print_sys(
                            "The input is a list, docking score will be computed! If you already have the docking scores, please make the list as a dictionary with SMILES string as key and docking score as value"
                        )
                    else:
                        raise ValueError(
                            "The input prediction must be a dictionary with SMILES and their docking scores or a list of SMILES!"
                        )
                    ## pred is a list of smiles strings or a dictionary of smiles strings if docking scores are already calculated...
                    if len(pred_) != 100:
                        raise ValueError(
                            "The expected output is a list/dictionary of top 100 molecules!"
                        )

                    if recalc:
                        dataset = fuzzy_search(data_name, self.dataset_names)

                        # docking scores for the top K smiles (K <= 100)
                        target_pdb_file = os.path.join(self.path,
                                                       dataset + ".pdb")
                        from .oracles import Oracle

                        data_path = os.path.join(self.path, dataset)
                        oracle = Oracle(
                            name="Docking_Score",
                            software="vina",
                            pyscreener_path=self.pyscreener_path,
                            receptors=[target_pdb_file],
                            center=docking_target_info[dataset]["center"],
                            size=docking_target_info[dataset]["size"],
                            buffer=10,
                            path=data_path,
                            num_worker=self.num_workers,
                            ncpu=self.num_cpus,
                            num_max_call=10000,
                        )

                        docking_scores = oracle(pred_)
                    print_sys("---- Calculating average docking scores ----")
                    if (len(
                            np.where(
                                np.array(list(docking_scores.values())) > 0)[0])
                            > 0.7):
                        ## check if the scores are all positive.. if so, make them all negative
                        docking_scores = {
                            j: -k for j, k in docking_scores.items()
                        }
                    if save_dict:
                        results["docking_scores_dict"] = docking_scores
                    values = np.array(list(docking_scores.values()))
                    results["top100"] = np.mean(values)
                    results["top10"] = np.mean(sorted(values)[:10])
                    results["top1"] = min(values)

                    if m1_api is None:
                        print_sys(
                            "Ignoring M1 Synthesizability Evaluations. You can still submit your results without m1 score. Although for the submission, we encourage inclusion of m1 scores. To opt-in, set the m1_api to the token obtained via: https://tdcommons.ai/functions/oracles/#moleculeone"
                        )
                    else:
                        print_sys(
                            "---- Calculating molecule.one synthesizability score ----"
                        )
                        from .oracles import Oracle

                        m1 = Oracle(name="Molecule One Synthesis",
                                    api_token=m1_api)
                        import heapq
                        from operator import itemgetter

                        top10_docking_smiles = list(
                            dict(
                                heapq.nsmallest(10,
                                                docking_scores.items(),
                                                key=itemgetter(1))).keys())
                        m1_scores = m1(top10_docking_smiles)
                        scores_array = list(m1_scores.values())
                        scores_array = np.array(
                            [float(i) for i in scores_array])
                        scores_array[np.where(
                            scores_array == -1.0
                        )[0]] = 10  # m1 score errors are usually large complex molecules
                        if save_dict:
                            results["m1_dict"] = m1_scores
                        results["m1"] = np.mean(scores_array)

                    print_sys("---- Calculating molecular filters scores ----")
                    from .chem_utils import MolFilter

                    ## follow guacamol
                    filters = MolFilter(
                        filters=["PAINS", "SureChEMBL", "Glaxo"],
                        property_filters_flag=False,
                    )
                    pred_filter = filters(pred_)
                    if save_dict:
                        results["pass_list"] = pred_filter
                    results["%pass"] = float(len(pred_filter)) / 100
                    results["top1_%pass"] = max(
                        [docking_scores[i] for i in pred_filter])
                    print_sys("---- Calculating diversity ----")
                    from .evaluator import Evaluator

                    evaluator = Evaluator(name="Diversity")
                    score = evaluator(pred_)
                    results["diversity"] = score
                    print_sys("---- Calculating novelty ----")
                    evaluator = Evaluator(name="Novelty")
                    training = pd.read_csv(os.path.join(self.path, "zinc.tab"),
                                           sep="\t")
                    score = evaluator(pred_, training.smiles.values)
                    results["novelty"] = score
                    results["top smiles"] = [
                        i[0] for i in sorted(docking_scores.items(),
                                             key=lambda x: x[1])
                    ]
                    results_max_call[num_max_call] = results
                results_all[data_name] = results_max_call
            return results_all

        if true is None:
            # test set evaluation
            metric_dict = bm_metric_names[self.name]
            out = {}
            for data_name, pred_ in pred.items():
                data_name = fuzzy_search(data_name, self.dataset_names)
                data_path = os.path.join(self.path, data_name)
                if self.file_format == "csv":
                    test = pd.read_csv(os.path.join(data_path, "test.csv"))
                elif self.file_format == "pkl":
                    test = pd.read_pickle(os.path.join(data_path, "test.pkl"))
                y = test.Y.values
                evaluator = eval("Evaluator(name = '" + metric_dict[data_name] +
                                 "')")
                out[data_name] = {
                    metric_dict[data_name]: round(evaluator(y, pred_), 3)
                }

                # If reporting accuracy across target classes
                if "target_class" in test.columns:
                    test["pred"] = pred_
                    for c in test["target_class"].unique():
                        data_name_subset = data_name + "_" + c
                        test_subset = test[test["target_class"] == c]
                        y_subset = test_subset.Y.values
                        pred_subset = test_subset.pred.values

                        evaluator = eval("Evaluator(name = '" +
                                         metric_dict[data_name_subset] + "')")
                        out[data_name_subset] = {
                            metric_dict[data_name_subset]:
                                round(evaluator(y_subset, pred_subset), 3)
                        }
            return out
        else:
            # validation set evaluation
            if benchmark is None:
                raise ValueError(
                    "Please specify the benchmark name for us to retrieve the standard metric!"
                )
            data_name = fuzzy_search(benchmark, self.dataset_names)
            metric_dict = bm_metric_names[self.name]
            evaluator = eval("Evaluator(name = '" + metric_dict[data_name] +
                             "')")
            return {metric_dict[data_name]: round(evaluator(true, pred), 3)}

    def evaluate_many(self,
                      preds,
                      save_file_name=None,
                      m1_api=None,
                      results_individual=None):
        """
        :param preds: list of dict<str dataset_name: list of float>
        :return: dict<dataset_name: [mean_metric_result, std_metric_result]

        This function returns the data in a format needed to submit to the Leaderboard
        """

        if self.name == "docking_group":
            min_requirement = 3
        else:
            min_requirement = 5

        if len(preds) < min_requirement:
            return ValueError("Must have predictions from at least " +
                              str(min_requirement) +
                              " runs for leaderboard submission")
        if results_individual is None:
            individual_results = []
            for pred in preds:
                retval = self.evaluate(pred, m1_api=m1_api)
                individual_results.append(retval)
        else:
            individual_results = results_individual

        if self.name == "docking_group":
            metrics = [
                "top100",
                "top10",
                "top1",
                "diversity",
                "novelty",
                "%pass",
                "top1_%pass",
                "m1",
                "top smiles",
            ]
            num_folds = len(preds)

            results_agg = {}

            for target in list(individual_results[0].keys()):
                results_agg_target = {}
                for num_calls in individual_results[0][target].keys():
                    results_agg_target_call = {}
                    for metric in metrics:
                        if metric == "top smiles":
                            results_agg_target_call[metric] = np.unique(
                                np.array([
                                    individual_results[fold][target][num_calls]
                                    [metric] for fold in range(num_folds)
                                ]).reshape(-1)).tolist()
                        else:
                            res = [
                                individual_results[fold][target][num_calls]
                                [metric] for fold in range(num_folds)
                            ]
                            results_agg_target_call[metric] = [
                                round(np.mean(res), 3),
                                round(np.std(res), 3),
                            ]
                    results_agg_target[num_calls] = results_agg_target_call
                results_agg[target] = results_agg_target

            import pickle

            if save_file_name is None:
                save_file_name = "tdc_docking_result"
            with open(save_file_name + ".pkl", "wb") as f:
                pickle.dump(results_agg, f)
            return results_agg

        else:
            given_dataset_names = list(individual_results[0].keys())
            aggregated_results = {}
            for dataset_name in given_dataset_names:
                my_results = []
                for individual_result in individual_results:
                    my_result = list(
                        individual_result[dataset_name].values())[0]
                    my_results.append(my_result)
                u = np.mean(my_results)
                std = np.std(my_results)
                aggregated_results[dataset_name] = [round(u, 3), round(std, 3)]
            return aggregated_results
